#!/usr/bin/env python
# --!-- coding: utf8 --!--
import re

from PyQt5.QtGui import QTextCharFormat, QFont
from PyQt5.QtWidgets import qApp, QVBoxLayout, QCheckBox, QWidget, QHBoxLayout, QLabel, QSpinBox, QComboBox

from manuskript.exporter.manuskript.markdown import markdown, markdownSettings
from manuskript.ui.collapsibleGroupBox2 import collapsibleGroupBox2
from manuskript.functions import safeTranslate

import logging
LOGGER = logging.getLogger(__name__)


class abstractPlainText(markdown):
    name = "SUBCLASSME"
    description = "SUBCLASSME"
    exportVarName = "SUBCLASSME"
    toFormat = "SUBCLASSME"
    icon = "SUBCLASSME"
    exportFilter = "SUBCLASSME"
    exportDefaultSuffix = ".SUBCLASSME"

    def __init__(self, exporter):
        self.exporter = exporter

    def settingsWidget(self):
        # Get pandoc major version to determine valid command line options
        p = re.compile(r'pandoc (\d+)\.(\d+).*')
        m = p.match(self.exporter.version())
        if m:
            majorVersion = m.group(1)
            minorVersion = m.group(2)
        else:
            majorVersion = ""
            minorVersion = ""
        w = pandocSettings(self, majorVersion, minorVersion, toFormat=self.toFormat)
        w.loadSettings()
        return w

    def src(self, settingsWidget):
        return markdown.output(self, settingsWidget)

    def output(self, settingsWidget, outputfile=None):
        args = settingsWidget.runnableSettings()
        src = self.src(settingsWidget)
        return self.exporter.convert(src, args, outputfile)

    def preview(self, settingsWidget, previewWidget):
        settings = settingsWidget.getSettings()

        # Save settings
        settingsWidget.writeSettings()

        # Prepares text edit
        self.preparesTextEditViewMarkdown(previewWidget, settingsWidget.settings)
        self.preparesTextEditView(previewWidget, settings["Preview"]["PreviewFont"])

        r = self.output(settingsWidget)

        previewWidget.setPlainText(r)


def versionAsInt(version):
    if version is None:
        return 0

    try:
        return int(version)
    except ValueError:
        return 0


def versionToIntArray(version):
    if version is None:
        return [0, 0]

    p = re.compile(r'(\d+)\.(\d+).*')
    m = p.match(version)
    if m:
        majorVersion = m.group(1)
        minorVersion = m.group(2)
    else:
        majorVersion = ""
        minorVersion = ""

    return [ versionAsInt(majorVersion), versionAsInt(minorVersion) ]


class pandocSetting:
    def __init__(self, arg, type, format, label, widget=None, default=None, min=None, max=None, vals=None, suffix="",
                 minVersion=None, maxVersion=None, specific=False, toc=False):
        self.arg = arg  # start with EXT for extensions
        self.type = type
        self.label = label
        self.formats = format
        if "html" in self.formats:
            self.formats += " slidy dzslides revealjs slideous s5 html5"
        self.widget = widget
        self.default = default
        self.min = min
        self.max = max
        self.vals = vals.split("|") if vals else []
        self.suffix = suffix
        self.minVersion = versionToIntArray(minVersion)
        self.maxVersion = versionToIntArray(maxVersion)
        self.specific = specific
        self.toc = toc

    def isValid(self, format):
        """Return whether the specific setting is active with the given format."""

        # Empty formats means all
        if self.formats == "":
            return True

        # "html" in "html markdown latex"
        elif format in self.formats:
            return True

        # "markdown_strict" in "html markdown latex"
        elif [f for f in self.formats.split(" ") if format in f]:
            return True

        return False

    def isCompatible(self, majorVersion, minorVersion):
        majorNumber = versionAsInt(majorVersion)
        minorNumber = versionAsInt(minorVersion)

        if (majorNumber < self.minVersion[0]) or ((majorNumber == self.minVersion[0]) and
                                                  (minorNumber < self.minVersion[1])):
            return False

        if (self.maxVersion[0] == 0) and (self.maxVersion[1] == 0):
            return True

        return (majorNumber < self.maxVersion[0]) or ((majorNumber == self.maxVersion[0]) and
                                                      (minorNumber <= self.maxVersion[1]))

    def isSpecific(self):
        return self.specific

    def isTOC(self):
        return self.toc


class pandocSettings(markdownSettings):

    settingsList = {
        # General
        "standalone": pandocSetting("--standalone", "checkbox", "",
                                    safeTranslate(qApp, "Export", "Standalone document (not just a fragment)"),
                                    default=True),
        "TOC":          pandocSetting("--toc", "checkbox", "",
                                      safeTranslate(qApp, "Export", "Include a table of contents."), toc=True),

        "TOC-depth":    pandocSetting("--toc-depth=", "number", "",
                                      safeTranslate(qApp, "Export", "Number of sections level to include in TOC: "),
                                      default=3, min=1, max=6, toc=True, minVersion="1.10"),
        # pandoc v1 only
        "smart":        pandocSetting("--smart", "checkbox", "",
                                      safeTranslate(qApp, "Export", "Typographically correct output"),
                                      maxVersion="1.19.2.4"),
        # pandoc v1 only
        "normalize":    pandocSetting("--normalize", "checkbox", "",
                                      safeTranslate(qApp, "Export", "Normalize the document (cleaner)"),
                                      minVersion="1.8", maxVersion="1.19.2.4"),
        # pandoc v1.5 to 2.7.3
        "base-header": pandocSetting("--base-header-level=", "number", "",
                                     safeTranslate(qApp, "Export", "Specify the base level for headers: "),
                                     default=1, min=1, minVersion="1.5", maxVersion="2.7.3"),
        # pandoc v2.8+
        "shift-heading":  pandocSetting("--shift-heading-level-by=", "number", "",
                                      safeTranslate(qApp, "Export", "Specify the base level for headers: "),
                                      default=0, min=0, minVersion="2.8"),
        "disable-YAML": pandocSetting("EXT-yaml_metadata_block", "checkbox", "",
                                      safeTranslate(qApp, "Export", "Disable YAML metadata block.\nUse that if you get YAML related error."),
                                      minVersion="1.12"),
        "hard-line-breaks": pandocSetting("EXT-hard_line_block", "checkbox", "",
                                      safeTranslate(qApp, "Export", "Enable the support on markdown for line break on new line."),
                                      minVersion="1.16"),

        # Specific
        "ref-link":     pandocSetting("--reference-links", "checkbox", "markdown rst",
                                      safeTranslate(qApp, "Export", "Use reference-style links instead of inline links"),
                                      specific=True),
        # pandoc v1.9 to v2.11.1
        "atx":          pandocSetting("--atx-headers", "checkbox", "markdown asciidoc",
                                      safeTranslate(qApp, "Export", "Use ATX-style headers"), specific=True,
                                      minVersion="1.9", maxVersion="2.11.1"),
        # pandoc v2.11.2+
        "atx-heading": pandocSetting("--markdown-headings=atx|setext", "checkbox", "markdown asciidoc",
                                     safeTranslate(qApp, "Export", "Use ATX-style headers"), specific=True,
                                     minVersion="2.11.2"),
        "self-contained": pandocSetting("--self-contained", "checkbox", "html",
                                        safeTranslate(qApp, "Export", "Self-contained HTML files, with no dependencies"),
                                        specific=True, minVersion="1.9"),
        "q-tags":       pandocSetting("--html-q-tags", "checkbox", "html",
                                      safeTranslate(qApp, "Export", "Use <q> tags for quotes in HTML"), specific=True,
                                      minVersion="1.10"),
        # pandoc v1 only
        "latex-engine": pandocSetting("--latex-engine=", "combo", "pdf",
                                      safeTranslate(qApp, "Export", "LaTeX engine used to produce the PDF."),
                                      vals="pdflatex|lualatex|xelatex", specific=True,
                                      minVersion="1.9", maxVersion="1.19.2.4"),
        # pandoc v2
        "pdf-engine":   pandocSetting("--pdf-engine=", "combo", "pdf",
                                      safeTranslate(qApp, "Export", "LaTeX engine used to produce the PDF."),
                                      vals="pdflatex|lualatex|xelatex", minVersion="2.0", specific=True),
        "epub3":        pandocSetting("EXTepub3", "checkbox", "epub",
                                      safeTranslate(qApp, "Export", "Convert to ePUB3"), specific=True,
                                      minVersion="1.10"),

        # PDF
        "latex-ps":     pandocSetting("--variable=papersize:", "combo", "pdf latex",  # FIXME: does not work with default template
                                      safeTranslate(qApp, "Export", "Paper size:"),
                                      vals="letter|A4|A5", specific=True, minVersion="1.4"),
        "latex-fs":     pandocSetting("--variable=fontsize:", "number", "pdf latex",  # FIXME: does not work with default template
                                      safeTranslate(qApp, "Export", "Font size:"),
                                      min=8, max=88, default=12, suffix="pt", specific=True, minVersion="1.4"),
        "latex-class":  pandocSetting("--variable=documentclass:", "combo", "pdf latex",
                                     safeTranslate(qApp, "Export", "Class:"),
                                     vals="article|report|book|memoir", specific=True, minVersion="1.4"),
        "latex-ls":     pandocSetting("--variable=linestretch:", "combo", "pdf latex",
                                     safeTranslate(qApp, "Export", "Line spacing:"),
                                     vals="1|1.25|1.5|2", specific=True, minVersion="1.4"),

        # FIXME: complete with http://pandoc.org/README.html#variables-for-latex
    }


    def __init__(self, _format, majorVersion="", minorVersion="", toFormat=None, parent=None):
        markdownSettings.__init__(self, _format, parent)

        self.format = toFormat
        self.majorVersion = majorVersion
        self.minorVersion = minorVersion

        dropSettings = []

        for key, setting in self.settingsList.items():
            if not setting.isCompatible(self.majorVersion, self.minorVersion):
                dropSettings.append(key)

        LOGGER.info(f'Using pandoc settings: {self.majorVersion}.{self.minorVersion}, dropping: {dropSettings}')

        for key in dropSettings:
            self.settingsList.pop(key, None)

        w = QWidget(self)
        w.setLayout(QVBoxLayout())
        self.grpPandocGeneral = self.collapsibleGroupBox(self.tr("General"), w)
        self.grpPandocSpecific = self.collapsibleGroupBox(self.tr("Custom settings for {}").format(self.format), w)
        self.grpPandocTOC = self.collapsibleGroupBox(self.tr("Table of Content"), w)

        for key, setting in self.settingsList.items():
            if setting.isTOC():
                self.addSettingsWidget(key, self.grpPandocTOC)
            elif setting.isSpecific():
                self.addSettingsWidget(key, self.grpPandocSpecific)
            else:
                self.addSettingsWidget(key, self.grpPandocGeneral)

        self.toolBox.insertItem(self.toolBox.count() - 1, w, "Pandoc")
        self.toolBox.layout().setSpacing(0)  # Not sure why this is needed, but hey...

        self.getSettings()

    def collapsibleGroupBox(self, title, parent):
        g = collapsibleGroupBox2(title=title)
        parent.layout().addWidget(g)
        g.setLayout(QVBoxLayout())
        return g

    def addSettingsWidget(self, settingsName, parent):

        if not settingsName in self.settingsList:
            return

        s = self.settingsList[settingsName]

        if not s.isValid(self.format):
            # That setting is not available for that export format
            return

        if "checkbox" in s.type:
            s.widget = QCheckBox(s.label)
            if s.default:
                s.widget.setChecked(True)
            parent.layout().addWidget(s.widget)

        elif "number" in s.type:
            l = QHBoxLayout()
            label = QLabel(s.label, parent)
            label.setWordWrap(True)
            l.addWidget(label, 8)
            s.widget = QSpinBox()
            s.widget.setValue(s.default if s.default else 0)
            if s.min:
                s.widget.setMinimum(s.min)
            if s.max:
                s.widget.setMaximum(s.max)
            if s.suffix:
                s.widget.setSuffix(s.suffix)
            l.addWidget(s.widget, 2)
            parent.layout().addLayout(l)

        elif "combo" in s.type:
            l = QHBoxLayout()
            label = QLabel(s.label, parent)
            label.setWordWrap(True)
            l.addWidget(label, 6)
            s.widget = QComboBox()
            s.widget.addItems(s.vals)
            l.addWidget(s.widget, 2)
            parent.layout().addLayout(l)

    def updateFromSettings(self):
        markdownSettings.updateFromSettings(self)

        # s = self.settings["Preview"]
        # val = s.get("MarkdownHighlighter", False)
        # self.chkMarkdownHighlighter.setChecked(val)

        if not "Pandoc" in self.settings:
            return

        for name in self.settingsList:
            s = self.settingsList[name]

            if s.isValid(self.format):
                if s.type == "checkbox" and name in self.settings["Pandoc"]:
                    s.widget.setChecked(self.settings["Pandoc"][name])
                elif s.type == "number" and name in self.settings["Pandoc"]:
                    s.widget.setValue(int(self.settings["Pandoc"][name]))
                elif s.type == "combo" and name in self.settings["Pandoc"]:
                    s.widget.setCurrentText(self.settings["Pandoc"][name])


    def getSettings(self):
        self.settings = markdownSettings.getSettings(self)

        P = self.settings.get("Pandoc", {})

        for name in self.settingsList:
            s = self.settingsList[name]

            if s.isValid(self.format):
                if s.type == "checkbox":
                    P[name] = s.widget.isChecked()
                elif s.type == "number":
                    P[name] = str(s.widget.value())
                elif s.type == "combo":
                    P[name] = s.widget.currentText()

        self.settings["Pandoc"] = P

        return self.settings

    def runnableSettings(self):

        # First we get extensions (where arg starts with EXT)
        extensions = ""
        toFormat = self.format
        for name in self.settingsList:
            s = self.settingsList[name]
            if s.arg[:3] == "EXT" and s.isValid(self.format):
                if name == "disable-YAML" and s.widget.isChecked():
                    extensions += "-yaml_metadata_block"
                if name == "epub3" and s.widget.isChecked():
                    toFormat = "epub3"
                if name == "hard-line-breaks" and s.widget.isChecked():
                    extensions += "+hard_line_breaks"

        r = ["--from=markdown" + extensions,
             "--to={}".format(toFormat)]

        # Add every command
        for name in self.settingsList:
            s = self.settingsList[name]

            if s.arg[:3] == "EXT":
                continue

            if s.isValid(self.format):
                rr = ""
                if s.type == "checkbox":
                    if s.widget.isChecked():
                        rr = s.arg
                elif s.type == "number":
                    rr = "{}{}".format(
                        s.arg,
                        str(s.widget.value())
                    )
                elif s.type == "combo":
                    rr = "{}{}".format(
                        s.arg,
                        s.widget.currentText()
                    )

                if rr:
                    r.append(rr+s.suffix)
        return r




#!/bin/python

import re
import sys

from lxml import etree


def main(argv) -> int:
    if len(argv) < 2:
        print("You need to specify a .ts file!")
        return 1

    path = argv[1]

    if (len(path) < 3) or (path[-3:] != ".ts"):
        print("Please specify a path to a .ts file!")
        return 2

    tree = None

    with open(path, "rb") as file:
        tree = etree.parse(file)

    if tree is None:
        print("Parsing failed!")
        return 3

    root = tree.getroot()
    if root.tag != "TS":
        print("Wrong type of file!")
        return 4

    for context in root.getchildren():
        if context.tag != "context":
            continue

        for message in context.getchildren():
            if message.tag != "message":
                continue

            source = message.find("source")
            translation = message.find("translation")

            if (source is None) or (translation is None):
                continue

            sourceText = etree.tostring(source, encoding="unicode").strip()

            if "&amp;" in sourceText:
                continue

            translationText = etree.tostring(translation, encoding="unicode").strip()
            translationText = re.sub(r"&amp;([a-zA-Z]+);", r"&\g<1>;", translationText)

            translationNode = etree.fromstring(translationText)
            translation.text = translationNode.text

    with open(path, "wb") as file:
        tree.write(file, encoding="utf-8", xml_declaration=True, pretty_print=True)

    print("Fixing finished!")
    return 0


if __name__ == "__main__":
    sys.exit(main(sys.argv))



import locale

from PyQt5.QtGui import QIcon, QPixmap
from PyQt5.QtWidgets import QWidget
from PyQt5.QtCore import QTimer

from manuskript.enums import Outline
from manuskript.functions import appPath
from manuskript.ui.tools.targets_ui import Ui_targets


class TargetsDialog(QWidget, Ui_targets):

    def __init__(self, parent=None, mw=None):
        QWidget.__init__(self)
        self.mw = parent
        self.setupUi(self)
        iconPic = appPath("icons/Manuskript/icon-64px.png")
        self.setWindowIcon(QIcon(iconPic))

        self.session_reset.clicked.connect(self.resetSession)
        
        self.tick()

        self.timer = QTimer()
        self.timer.timeout.connect(self.tick)
        self.timer.start(2000)

    def getDraftStats(self):
        item = self.mw.mdlOutline.rootItem

        wc = item.data(Outline.wordCount)
        goal = item.data(Outline.goal)
        progress = item.data(Outline.goalPercentage)

        return (
            int(wc) if wc != "" else 0,
            int(goal) if goal != "" else 0,
            float(progress) if progress != "" else 0.0
        )

    def resetSession(self):
        wc, _, _ = self.getDraftStats()
        self.mw.sessionStartWordCount = wc 
        self.tick()

    @staticmethod
    def progress_bar_value(value):
        return int(min(max(float(value) * 100, 0), 100))

    def tick(self):
        wc, goal, progress = self.getDraftStats()
        
        self.draft_wc_label.setText(locale.format_string("%d", wc, grouping=True))
        self.draft_goal_label.setText(locale.format_string("%d", goal, grouping=True))
        # limit to 0-100 for display
        self.draft_progress_bar.setValue(self.progress_bar_value(progress))

        session_wc = wc - int(self.mw.sessionStartWordCount)

        self.session_wc_label.setText(locale.format_string("%d", session_wc, grouping=True))
        if self.session_target.value() == 0:
            self.session_progress_bar.setValue(0)
        else:
            self.session_progress_bar.setValue(self.progress_bar_value(session_wc / self.session_target.value()))

    def closeEvent(self, event):
        self.timer = None



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript\manuskript\ui\bulkInfoManager.ui'
#
# Created by: PyQt5 UI code generator 5.15.4
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_BulkInfoManager(object):
    def setupUi(self, BulkInfoManager):
        BulkInfoManager.setObjectName("BulkInfoManager")
        BulkInfoManager.setWindowModality(QtCore.Qt.WindowModal)
        BulkInfoManager.resize(644, 556)
        self.verticalLayout = QtWidgets.QVBoxLayout(BulkInfoManager)
        self.verticalLayout.setObjectName("verticalLayout")
        self.lblStaticMessage = QtWidgets.QLabel(BulkInfoManager)
        self.lblStaticMessage.setObjectName("lblStaticMessage")
        self.verticalLayout.addWidget(self.lblStaticMessage)
        self.lblCharactersDynamic = QtWidgets.QLabel(BulkInfoManager)
        self.lblCharactersDynamic.setObjectName("lblCharactersDynamic")
        self.verticalLayout.addWidget(self.lblCharactersDynamic)
        self.tableView = QtWidgets.QTableView(BulkInfoManager)
        self.tableView.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedKingdom))
        self.tableView.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
        self.tableView.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.tableView.setAlternatingRowColors(True)
        self.tableView.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableView.setTextElideMode(QtCore.Qt.ElideNone)
        self.tableView.setCornerButtonEnabled(False)
        self.tableView.setObjectName("tableView")
        self.tableView.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.tableView)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.btnPersoBulkAddInfo = QtWidgets.QPushButton(BulkInfoManager)
        self.btnPersoBulkAddInfo.setText("")
        icon = QtGui.QIcon.fromTheme("list-add")
        self.btnPersoBulkAddInfo.setIcon(icon)
        self.btnPersoBulkAddInfo.setObjectName("btnPersoBulkAddInfo")
        self.horizontalLayout.addWidget(self.btnPersoBulkAddInfo)
        self.btnPersoBulkRmInfo = QtWidgets.QPushButton(BulkInfoManager)
        self.btnPersoBulkRmInfo.setText("")
        icon = QtGui.QIcon.fromTheme("list-remove")
        self.btnPersoBulkRmInfo.setIcon(icon)
        self.btnPersoBulkRmInfo.setObjectName("btnPersoBulkRmInfo")
        self.horizontalLayout.addWidget(self.btnPersoBulkRmInfo)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.btnPersoBulkApply = QtWidgets.QPushButton(BulkInfoManager)
        self.btnPersoBulkApply.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedKingdom))
        self.btnPersoBulkApply.setObjectName("btnPersoBulkApply")
        self.horizontalLayout.addWidget(self.btnPersoBulkApply)
        self.horizontalLayout.setStretch(0, 1)
        self.horizontalLayout.setStretch(1, 1)
        self.horizontalLayout.setStretch(2, 5)
        self.horizontalLayout.setStretch(3, 1)
        self.verticalLayout.addLayout(self.horizontalLayout)

        self.retranslateUi(BulkInfoManager)
        QtCore.QMetaObject.connectSlotsByName(BulkInfoManager)

    def retranslateUi(self, BulkInfoManager):
        _translate = QtCore.QCoreApplication.translate
        BulkInfoManager.setWindowTitle(_translate("BulkInfoManager", "Form"))
        self.lblStaticMessage.setText(_translate("BulkInfoManager", "Affected Characters:"))
        self.lblCharactersDynamic.setText(_translate("BulkInfoManager", "NONE"))
        self.btnPersoBulkAddInfo.setToolTip(_translate("BulkInfoManager", "Add entry to the list."))
        self.btnPersoBulkRmInfo.setToolTip(_translate("BulkInfoManager", "Remove entry from the list."))
        self.btnPersoBulkApply.setToolTip(_translate("BulkInfoManager", "Adds all items to the selected characters."))
        self.btnPersoBulkApply.setText(_translate("BulkInfoManager", "Apply Changes"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    BulkInfoManager = QtWidgets.QWidget()
    ui = Ui_BulkInfoManager()
    ui.setupUi(BulkInfoManager)
    BulkInfoManager.show()
    sys.exit(app.exec_())



#!/usr/bin/env python
# --!-- coding: utf8 --!--
from PyQt5.QtCore import QSize, QModelIndex, Qt
from PyQt5.QtGui import QPixmap, QColor, QIcon, QBrush
from PyQt5.QtWidgets import QTreeWidget, QTreeWidgetItem, QColorDialog, QDialog, QMessageBox

from manuskript.enums import Character
from manuskript.functions import iconColor, mainWindow
from manuskript.ui import style as S
from manuskript.ui import characterInfoDialog


class characterTreeView(QTreeWidget):
    """
    A QTreeWidget that displays characters from a characterModel in respect of their importance.
    """
    def __init__(self, parent=None):
        QTreeWidget.__init__(self, parent)
        self._model = None
        self._catRow = [-1, -1, -1]
        self._filter = ""
        self._lastID = -1
        self._updating = False
        self.setRootIsDecorated(False)
        self.setIndentation(10)
        self.setHeaderHidden(True)
        self.setIconSize(QSize(24, 24))

        self.setColumnCount(1)
        self._rootItem = QTreeWidgetItem()
        self.insertTopLevelItem(0, self._rootItem)

        self.importanceMap = {self.tr("Main"):2, self.tr("Secondary"):1, self.tr("Minor"):0}

    def setCharactersModel(self, model):
        self._model = model
        self._model.dataChanged.connect(self.updateMaybe)
        self._model.rowsInserted.connect(self.updateMaybe2)
        self._model.rowsRemoved.connect(self.updateMaybe2)
        self.updateItems()

    def setFilter(self, text):
        self._filter = text
        self.updateItems()

    def updateMaybe(self, topLeft, bottomRight):
        if topLeft.parent() != QModelIndex():
            return

        if topLeft.column() <= Character.name <= bottomRight.column():
            # Update name
            self.updateNames()

        elif topLeft.column() <= Character.importance <= bottomRight.column():
            # Importance changed
            self.updateItems()

    def updateMaybe2(self, parent, first, last):
        # Rows inserted or removed, we update only if they are topLevel rows.
        if parent == QModelIndex():
            self.updateItems()

    def updateNames(self):
        for i in range(self.topLevelItemCount()):
            item = self.topLevelItem(i)

            for child in range(item.childCount()):
                sub = item.child(child)
                ID = sub.data(0, Qt.UserRole)
                if ID != None:
                    # Update name
                    c = self._model.getCharacterByID(ID)
                    name = c.name()
                    sub.setText(0, name)
                    # Update icon
                    px = QPixmap(32, 32)
                    color = c.color()
                    px.fill(color)
                    sub.setIcon(0, QIcon(px))

    def updateItems(self):
        if not self._model:
            return

        if self.currentItem():
            self._lastID = self.currentItem().data(0, Qt.UserRole)

        self._updating = True
        self.clear()
        characters = self._model.getCharactersByImportance()

        for i, importanceLevel in enumerate(self.importanceMap):
            # Create category item
            cat = QTreeWidgetItem(self, [importanceLevel])
            cat.setBackground(0, QBrush(QColor(S.highlightLight)))
            cat.setForeground(0, QBrush(QColor(S.highlightedTextDark)))
            cat.setTextAlignment(0, Qt.AlignCenter)
            f = cat.font(0)
            f.setBold(True)
            cat.setFont(0, f)
            self.addTopLevelItem(cat)
            # cat.setChildIndicatorPolicy(cat.DontShowIndicator)

            for c in characters[i]:
                name = c.name()
                # Check if name passes filter
                if not self._filter.lower() in name.lower():
                    continue

                item = QTreeWidgetItem(cat, [name])
                item.setData(0, Qt.UserRole, c.ID())
                px = QPixmap(32, 32)
                color = QColor(c.color())
                px.fill(color)
                item.setIcon(0, QIcon(px))

                if c.ID() == self._lastID:
                    self.setCurrentItem(item)

        self.expandAll()
        self._updating = False

    def addCharacter(self):
        curr_item = self.currentItem()
        curr_importance = 0

        # check if an item is selected
        if curr_item != None:
            if curr_item.parent() == None:
                # this is a top-level category, so find its importance
                # get the current text, then look up the importance level
                text = curr_item.text(0)
                curr_importance = self.importanceMap[text]
            else:
                # get the importance from the currently-highlighted character
                curr_character = self.currentCharacter()
                curr_importance = curr_character.importance()

        self._model.addCharacter(importance=curr_importance)

    def removeCharacters(self):
        """
        Removes selected characters.
        """
        IDs = self.currentCharacterIDs()

        # If none of the IDs are valid, do nothing.
        if not any(IDs):
            return None

        #Get confirmation from user
        confirm = QMessageBox.warning(
            self, "Delete selected character(s)?",
            "Are you sure you want to delete the selected character(s)?",
            QMessageBox.Yes | QMessageBox.No
        )
        if confirm != QMessageBox.Yes:
            return None

        #Delete all selected characters
        for ID in IDs:
            self._model.removeCharacter(ID)
        return IDs

    def choseCharacterColor(self):
        ID = self.currentCharacterID()
        c = self._model.getCharacterByID(ID)

        if c:
            color = iconColor(c.icon)
        else:
            color = Qt.white

        self.colorDialog = QColorDialog(color, mainWindow())
        color = self.colorDialog.getColor(color)

        if color.isValid():
            c.setColor(color)
            mainWindow().updateCharacterColor(ID)

    def changeCharacterPOVState(self, state):
        ID = self.currentCharacterID()
        c = self._model.getCharacterByID(ID)
        c.setPOVEnabled(state == Qt.Checked)
        mainWindow().updateCharacterPOVState(ID)

    def addCharacterInfo(self):
        #Setting up dialog
        charInfoDialog = QDialog()
        charInfoUi = characterInfoDialog.Ui_characterInfoDialog()
        charInfoUi.setupUi(charInfoDialog)

        if charInfoDialog.exec_() == QDialog.Accepted:
            # User clicked OK, get the input values
            description = charInfoUi.descriptionLineEdit.text()
            value = charInfoUi.valueLineEdit.text()

            # Add the character info with the input values
            ID = self.currentCharacterID()
            self._model.addCharacterInfo(ID, description, value)



    def removeCharacterInfo(self):
        self._model.removeCharacterInfo(self.currentCharacterID())

    def currentCharacterID(self):
        ID = None
        if self.currentItem():
            ID = self.currentItem().data(0, Qt.UserRole)

        return ID

    def currentCharacterIDs(self): #Exactly like currentCharacterID(), except for multiselection
        IDs = []
        for item in self.selectedItems():
            ID = item.data(0, Qt.UserRole)
            if ID is not None:
                IDs.append(ID)
        return IDs

    def currentCharacter(self):
        """
        Returns the selected character
        @return: Character
        """
        ID = self.currentCharacterID()
        return self._model.getCharacterByID(ID)
    def currentCharacters(self):
        """
        Returns the selected characters (when multiple are selected)
        @return: List of Characters
        """
        IDs = self.currentCharacterIDs()
        characters = []
        for ID in IDs:
            characters.append(self._model.getCharacterByID(ID))
        return characters

    def getItemByID(self, ID):
        for t in range(self.topLevelItemCount()):
            for i in range(self.topLevelItem(t).childCount()):
                item = self.topLevelItem(t).child(i)
                if item.data(0, Qt.UserRole) == ID:
                    return item

    def mouseDoubleClickEvent(self, event):
        item = self.currentItem()
        if item is None:
            return
        # Catching double clicks to forbid collapsing of toplevel items
        if item.parent():
            QTreeWidget.mouseDoubleClickEvent(self, event)



#!/usr/bin/env python
# --!-- coding: utf8 --!--
import re

from PyQt5.QtCore import Qt, QRect, QEvent, QCoreApplication
from PyQt5.QtGui import QPalette, QFontMetrics, QKeySequence
from PyQt5.QtWidgets import QWidget, qApp, QListWidgetItem, QStyledItemDelegate, QStyle, QLabel, QToolTip, QShortcut


from manuskript.functions import mainWindow
from manuskript.ui import style
from manuskript.ui.search_ui import Ui_search
from manuskript.enums import Model

from manuskript.models.flatDataModelWrapper import flatDataModelWrapper
from manuskript.ui.searchMenu import searchMenu
from manuskript.ui.highlighters.searchResultHighlighters.searchResultHighlighter import searchResultHighlighter
import logging
LOGGER = logging.getLogger(__name__)


class search(QWidget, Ui_search):
    def __init__(self, parent=None):
        _translate = QCoreApplication.translate

        QWidget.__init__(self, parent)
        self.setupUi(self)

        self.searchTextInput.returnPressed.connect(self.search)
        self.searchTextInput.textChanged.connect(self.updateSearchFeedback)

        self.searchMenu = searchMenu()
        self.btnOptions.setMenu(self.searchMenu)
        self.searchMenu.triggered.connect(self.onSearchMenuChange)

        self.delegate = listResultDelegate(self)
        self.result.setItemDelegate(self.delegate)
        self.result.setMouseTracking(True)
        self.result.itemClicked.connect(self.openItem)

        self.result.setStyleSheet(style.searchResultSS())
        self.searchTextInput.setStyleSheet(style.lineEditSS())

        self.searchResultHighlighter = searchResultHighlighter()

        self.noResultsLabel = QLabel(_translate("Search", "No results found"), self.result)
        self.noResultsLabel.setVisible(False)
        self.noResultsLabel.setStyleSheet("QLabel {color: gray;}")

        # Add shortcuts for navigating through search results
        QShortcut(QKeySequence(_translate("MainWindow", "F3")), self.searchTextInput, self.nextSearchResult)
        QShortcut(QKeySequence(_translate("MainWindow", "Shift+F3")), self.searchTextInput, self.previousSearchResult)

        # These texts are already included in translation files but including ":" at the end. We force here the
        # translation for them without ":"
        _translate("MainWindow", "Situation")
        _translate("MainWindow", "Status")

    def nextSearchResult(self):
        if self.result.currentRow() < self.result.count() - 1:
            self.result.setCurrentRow(self.result.currentRow() + 1)
        else:
            self.result.setCurrentRow(0)

        if 0 < self.result.currentRow() < self.result.count():
            self.openItem(self.result.currentItem())

    def previousSearchResult(self):
        if self.result.currentRow() > 0:
            self.result.setCurrentRow(self.result.currentRow() - 1)
        else:
            self.result.setCurrentRow(self.result.count() - 1)

        if 0 < self.result.currentRow() < self.result.count():
            self.openItem(self.result.currentItem())

    def onSearchMenuChange(self):
        search_string = self.searchTextInput.text()
        self.updateSearchFeedback(search_string)

    def updateSearchFeedback(self, search_string):
        palette = QPalette()
        try:
            self.compileRegex(search_string)
        except Exception as e:
            # From https://stackoverflow.com/questions/27432456/python-qlineedit-text-color
            palette.setColor(QPalette.Text, Qt.red)

        self.searchTextInput.setPalette(palette)

    def prepareRegex(self, searchText):
        rtn = None
        try:
            rtn = self.compileRegex(searchText)
        except re.error as e:
            LOGGER.info("Problem preparing regular expression: " + e.msg)
            rtn = None
        except Exception as e:
            LOGGER.info("Problem preparing regular expression")
            rtn = None
        return rtn

    def compileRegex(self, searchText):
        # Intentionally throws exceptions for use elsewhere
        flags = re.UNICODE

        if self.searchMenu.caseSensitive() is False:
            flags |= re.IGNORECASE

        if self.searchMenu.regex() is False:
            searchText = re.escape(searchText)

        if self.searchMenu.matchWords() is True:
            # Source: https://stackoverflow.com/a/15863102
            searchText = r'\b' + searchText + r'\b'

        return re.compile(searchText, flags)

    def search(self):
        self.result.clear()
        self.result.setCurrentRow(0)

        searchText = self.searchTextInput.text()
        if len(searchText) > 0:
            results = list()
            searchRegex = self.prepareRegex(searchText)
            if searchRegex is not None:
                # Set override cursor
                qApp.setOverrideCursor(Qt.WaitCursor)

                for model, modelName in [
                    (mainWindow().mdlOutline, Model.Outline),
                    (mainWindow().mdlCharacter, Model.Character),
                    (flatDataModelWrapper(mainWindow().mdlFlatData), Model.FlatData),
                    (mainWindow().mdlWorld, Model.World),
                    (mainWindow().mdlPlots, Model.Plot)
                ]:
                    filteredColumns = self.searchMenu.columns(modelName)

                    # Searching
                    if len(filteredColumns):
                        results += model.searchOccurrences(searchRegex, filteredColumns)

                # Showing results
                self.generateResultsLists(results)

                # Remove override cursor
                qApp.restoreOverrideCursor()
            else:
                # No results to generate if there is a problem with the regex
                self.generateResultsLists(list())

    def generateResultsLists(self, results):
        self.noResultsLabel.setVisible(len(results) == 0)
        for result in results:
            item = QListWidgetItem(result.title(), self.result)
            item.setData(Qt.UserRole, result)
            item.setData(Qt.UserRole + 1, ' > '.join(result.path()))
            item.setData(Qt.UserRole + 2, result.context())
            self.result.addItem(item)

    def openItem(self, item):
        self.searchResultHighlighter.highlightSearchResult(item.data(Qt.UserRole))

    def leaveEvent(self, event):
        self.delegate.mouseLeave()


class listResultDelegate(QStyledItemDelegate):
    def __init__(self, parent=None):
        QStyledItemDelegate.__init__(self, parent)
        self._tooltipRowIndex = -1

    def paint(self, painter, option, index):
        extra = index.data(Qt.UserRole + 1)

        if not extra:
            return QStyledItemDelegate.paint(self, painter, option, index)
        else:
            if option.state & QStyle.State_Selected:
                painter.fillRect(option.rect, option.palette.color(QPalette.Highlight))

            title = index.data()
            painter.drawText(option.rect.adjusted(2, 1, 0, 0), Qt.AlignLeft, title)

            fm = QFontMetrics(option.font)
            w = fm.width(title)
            r = QRect(option.rect)
            r.setLeft(r.left() + w)
            painter.save()
            if option.state & QStyle.State_Selected:
                painter.setPen(Qt.white)
            else:
                painter.setPen(Qt.gray)
            painter.drawText(r.adjusted(2, 1, 0, 0), Qt.AlignLeft, " - {}".format(extra))
            painter.restore()

    def editorEvent(self, event, model, option, index):
        if event.type() == QEvent.MouseMove and self._tooltipRowIndex != index.row():
            self._tooltipRowIndex = index.row()
            context = index.data(Qt.UserRole + 2)
            extra = index.data(Qt.UserRole + 1)
            QToolTip.showText(event.globalPos(),
                              "<p>#" + str(index.row()) + " - " + extra + "</p><p>" + context + "</p>")
            return True
        return False

    def mouseLeave(self):
        self._tooltipRowIndex = -1



#!/usr/bin/env python
# --!-- coding: utf8 --!--
from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import QTreeView, QHeaderView

from manuskript import settings
from manuskript.enums import Outline
from manuskript.ui.views.dndView import dndView
from manuskript.ui.views.outlineBasics import outlineBasics
from manuskript.ui.views.outlineDelegates import outlineTitleDelegate, outlineCharacterDelegate, outlineCompileDelegate, \
    outlineStatusDelegate, outlineGoalPercentageDelegate, outlineLabelDelegate


class outlineView(QTreeView, dndView, outlineBasics):
    def __init__(self, parent=None, modelCharacters=None, modelLabels=None, modelStatus=None):
        QTreeView.__init__(self, parent)
        dndView.__init__(self)
        outlineBasics.__init__(self, parent)

        self.modelCharacters = modelCharacters
        self.modelLabels = modelLabels
        self.modelStatus = modelStatus

        self.header().setStretchLastSection(False)

    def setModelCharacters(self, model):
        # This is used by outlineCharacterDelegate to select character
        self.modelCharacters = model

    def setModelLabels(self, model):
        # This is used by outlineLabelDelegate to display labels
        self.modelLabels = model

    def setModelStatus(self, model):
        # This is used by outlineStatusDelegate to display statuses
        self.modelStatus = model

    def setModel(self, model):
        QTreeView.setModel(self, model)

        # Setting delegates
        self.outlineTitleDelegate = outlineTitleDelegate(self)
        # self.outlineTitleDelegate.setView(self)
        self.setItemDelegateForColumn(Outline.title, self.outlineTitleDelegate)
        self.outlineCharacterDelegate = outlineCharacterDelegate(self.modelCharacters)
        self.setItemDelegateForColumn(Outline.POV, self.outlineCharacterDelegate)
        self.outlineCompileDelegate = outlineCompileDelegate()
        self.setItemDelegateForColumn(Outline.compile, self.outlineCompileDelegate)
        self.outlineStatusDelegate = outlineStatusDelegate(self.modelStatus)
        self.setItemDelegateForColumn(Outline.status, self.outlineStatusDelegate)
        self.outlineGoalPercentageDelegate = outlineGoalPercentageDelegate()
        self.setItemDelegateForColumn(Outline.goalPercentage, self.outlineGoalPercentageDelegate)
        self.outlineLabelDelegate = outlineLabelDelegate(self.modelLabels)
        self.setItemDelegateForColumn(Outline.label, self.outlineLabelDelegate)

        # Hiding columns
        self.hideColumns()

        self.header().setSectionResizeMode(Outline.title, QHeaderView.ResizeToContents)
        self.header().setSectionResizeMode(Outline.POV, QHeaderView.ResizeToContents)
        self.header().setSectionResizeMode(Outline.status, QHeaderView.ResizeToContents)
        self.header().setSectionResizeMode(Outline.label, QHeaderView.ResizeToContents)
        self.header().setSectionResizeMode(Outline.compile, QHeaderView.ResizeToContents)
        self.header().setSectionResizeMode(Outline.wordCount, QHeaderView.ResizeToContents)
        self.header().setSectionResizeMode(Outline.goal, QHeaderView.ResizeToContents)
        self.header().setSectionResizeMode(Outline.goalPercentage, QHeaderView.ResizeToContents)

    def hideColumns(self):
        if not self.model():
            # outlineView is probably not initialized, because editorWidgets shows index cards or text.
            return

        for c in range(self.model().columnCount()):
            self.hideColumn(c)
        for c in settings.outlineViewColumns:
            self.showColumn(c)

    def setRootIndex(self, index):
        QTreeView.setRootIndex(self, index)
        self.outlineGoalPercentageDelegate = outlineGoalPercentageDelegate(index)
        self.setItemDelegateForColumn(Outline.goalPercentage, self.outlineGoalPercentageDelegate)

    def dragMoveEvent(self, event):
        dndView.dragMoveEvent(self, event)
        QTreeView.dragMoveEvent(self, event)

    def mousePressEvent(self, event):
        # Prevent selecting item while right-clicking for popup menu!
        if event.button() != Qt.RightButton:
            QTreeView.mousePressEvent(self, event)
            outlineBasics.mousePressEvent(self, event)

    def mouseReleaseEvent(self, event):
        QTreeView.mouseReleaseEvent(self, event)
        outlineBasics.mouseReleaseEvent(self, event)



#!/usr/bin/env python
# --!-- coding: utf8 --!--


from manuskript.models.searchResultModel import searchResultModel
from manuskript.functions import search
from PyQt5.QtCore import QCoreApplication


class searchableItem:

    def __init__(self, searchColumnLabels):
        self._searchColumnLabels = searchColumnLabels

    def searchOccurrences(self, searchRegex, column):
        return [self.wrapSearchOccurrence(column, startPos, endPos, context) for (startPos, endPos, context) in search(searchRegex, self.searchData(column))]

    def wrapSearchOccurrence(self, column, startPos, endPos, context):
        return searchResultModel(self.searchModel(), self.searchID(), column, self.searchTitle(column), self.searchPath(column), [(startPos, endPos)], context)

    def searchModel(self):
        raise NotImplementedError

    def searchID(self):
        raise NotImplementedError

    def searchTitle(self, column):
        raise NotImplementedError

    def searchPath(self, column):
        return []

    def searchData(self, column):
        raise NotImplementedError

    def searchColumnLabel(self, column):
        return self._searchColumnLabels.get(column, "")

    def translate(self, text):
        return QCoreApplication.translate("MainWindow", text)



#! /usr/bin/env python3
 
import sys
sys.path.insert(1, "/app/lib/manuskript")
print(sys.version)

# for debugging only
import PyQt5.QtWebEngineWidgets as webengine
print(webengine)

from manuskript import main
main.run()



#!/usr/bin/env python
# --!-- coding: utf8 --!--
from PyQt5.QtWidgets import QMenu, QAction
from PyQt5.QtCore import QCoreApplication
from PyQt5 import QtCore

from manuskript.searchLabels import OutlineSearchLabels, CharacterSearchLabels, FlatDataSearchLabels, WorldSearchLabels, PlotSearchLabels
from manuskript.models.searchFilter import searchFilter
from manuskript.enums import Model


def filterKey(modelPreffix, column):
    return modelPreffix + str(column)


class searchMenu(QMenu):
    def __init__(self, parent=None):
        QMenu.__init__(self, parent)

        _translate = QCoreApplication.translate
        # Model keys must match the ones used in search widget class
        self.filters = {
            Model.Outline: searchFilter(_translate("MainWindow", "Outline"), True, list(OutlineSearchLabels.keys())),
            Model.Character: searchFilter(_translate("MainWindow", "Characters"), True, list(CharacterSearchLabels.keys())),
            Model.FlatData: searchFilter(_translate("MainWindow", "FlatData"), True, list(FlatDataSearchLabels.keys())),
            Model.World: searchFilter(_translate("MainWindow", "World"), True, list(WorldSearchLabels.keys())),
            Model.Plot: searchFilter(_translate("MainWindow", "Plot"), True, list(PlotSearchLabels.keys()))
        }

        self.options = {
            "CS": [self.tr("Case sensitive"), True],
            "MatchWords": [self.tr("Match words"), False],
            "Regex": [self.tr("Regex"), False]
        }

        self._generateOptions()

    def _generateOptions(self):
        a = QAction(self.tr("Search in:"), self)
        a.setEnabled(False)
        self.addAction(a)
        for filterKey in self.filters:
            a = QAction(self.tr(self.filters[filterKey].label()), self)
            a.setCheckable(True)
            a.setChecked(self.filters[filterKey].enabled())
            a.setData(filterKey)
            a.triggered.connect(self._updateFilters)
            self.addAction(a)
        self.addSeparator()

        a = QAction(self.tr("Options:"), self)
        a.setEnabled(False)
        self.addAction(a)
        for optionKey in self.options:
            a = QAction(self.options[optionKey][0], self)
            a.setCheckable(True)
            a.setChecked(self.options[optionKey][1])
            a.setData(optionKey)
            a.triggered.connect(self._updateOptions)
            self.addAction(a)
        self.addSeparator()

    def _updateFilters(self):
        a = self.sender()
        self.filters[a.data()].setEnabled(a.isChecked())

    def _updateOptions(self):
        a = self.sender()
        self.options[a.data()][1] = a.isChecked()

    def columns(self, modelName):
        if self.filters[modelName].enabled():
            return self.filters[modelName].modelColumns()
        else:
            return []

    def caseSensitive(self):
        return self.options["CS"][1]

    def matchWords(self):
        return self.options["MatchWords"][1]

    def regex(self):
        return self.options["Regex"][1]

    def mouseReleaseEvent(self, event):
        # Workaround for enabling / disabling actions without closing the menu.
        # Source: https://stackoverflow.com/a/14967212
        action = self.activeAction()
        if action:
            action.setEnabled(False)
            QMenu.mouseReleaseEvent(self, event)
            action.setEnabled(True)
            action.trigger()
        else:
            QMenu.mouseReleaseEvent(self, event)

    def keyPressEvent(self, event):
        # Workaround for enabling / disabling actions without closing the menu.
        # Source: https://stackoverflow.com/a/14967212
        action = self.activeAction()
        if action and event.key() == QtCore.Qt.Key_Return:
            action.setEnabled(False)
            QMenu.keyPressEvent(self, event)
            action.setEnabled(True)
            action.trigger()
        else:
            QMenu.keyPressEvent(self, event)



#!/usr/bin/env python
# --!-- coding: utf8 --!--


from manuskript.models import references as Ref
from manuskript.functions import mainWindow
from manuskript.enums import Plot
from PyQt5.QtWidgets import QTextEdit, QLineEdit, QListView
from manuskript.ui.highlighters.searchResultHighlighters.abstractSpecificSearchResultHighlighter import abstractSearchResultHighlighter


class plotSearchResultHighlighter(abstractSearchResultHighlighter):
    def __init__(self):
        super().__init__()

    def openView(self, searchResult):
        r = Ref.plotReference(searchResult.id())
        Ref.open(r)
        mainWindow().tabPlot.setEnabled(True)

    def retrieveWidget(self, searchResult):
        textEditMap = {
            Plot.name: (0, "txtPlotName", QLineEdit),
            Plot.description: (0, "txtPlotDescription", QTextEdit),
            Plot.characters: (0, "lstPlotPerso", QListView),
            Plot.result: (0, "txtPlotResult", QTextEdit)
        }

        tabIndex, widgetName, widgetClass = textEditMap[searchResult.column()]

        mainWindow().tabPlot.setCurrentIndex(tabIndex)
        return mainWindow().tabPlot.findChild(widgetClass, widgetName)



#!/usr/bin/env python
# --!-- coding: utf8 --!--

from manuskript.models.abstractModel import abstractModel
from manuskript.models.searchableModel import searchableModel
from manuskript.models.outlineItem import outlineItem

class outlineModel(abstractModel, searchableModel):
    def __init__(self, parent):
        abstractModel.__init__(self, parent)
        self.rootItem = outlineItem(model=self, title="Root", ID="0")


    def findItemsByPOV(self, POV):
        "Returns a list of IDs of all items whose POV is ``POV``."
        return self.rootItem.findItemsByPOV(POV)

    def searchableItems(self):
        result = []

        for child in self.rootItem.children():
            result += self._searchableItems(child)

        return result

    def _searchableItems(self, item):
        result = [item]

        for child in item.children():
            result += self._searchableItems(child)

        return result



#!/usr/bin/env python
# --!-- coding: utf8 --!--
import os
import shutil
import subprocess

from PyQt5.QtCore import Qt
from PyQt5.QtWidgets import qApp, QMessageBox
from PyQt5.QtGui import QCursor

from manuskript.converters import abstractConverter
from manuskript.functions import mainWindow, safeTranslate

import logging
LOGGER = logging.getLogger(__name__)

class pandocConverter(abstractConverter):

    name = "pandoc"
    cmd = "pandoc"

    @classmethod
    def isValid(self):
        if self.path() != None:
            return 2
        elif self.customPath() and os.path.exists(self.customPath):
            return 1
        else:
            return 0

    @classmethod
    def customPath(self):
        settings = QSettings()
        return settings.value("Exporters/{}_customPath".format(self.name), "")

    @classmethod
    def path(self):
        return shutil.which(self.cmd)

    @classmethod
    def convert(self, src, _from="markdown", to="html", args=None, outputfile=None):
        if not self.isValid:
            LOGGER.error("pandocConverter is called but not valid.")
            return ""

        cmd = [self.runCmd()]

        cmd += ["--from={}".format(_from)]
        cmd += ["--to={}".format(to)]

        if args:
            cmd += args

        if outputfile:
            cmd.append("--output={}".format(outputfile))

        qApp.setOverrideCursor(QCursor(Qt.WaitCursor))

        p = subprocess.Popen(
            cmd,
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE
        )

        if not type(src) == bytes:
            src = src.encode("utf-8")  # assumes utf-8

        stdout, stderr = p.communicate(src)

        qApp.restoreOverrideCursor()

        if stderr:
            err = stderr.decode("utf-8")
            LOGGER.error(err)
            QMessageBox.critical(mainWindow().dialog,
                                 safeTranslate(qApp, "Export", "Error"), err)
            return None

        return stdout.decode("utf-8")

    @classmethod
    def runCmd(self):
        if self.isValid() == 2:
            return self.cmd
        elif self.isValid() == 1:
            return self.customPath



#!/usr/bin/env python
# --!-- coding: utf8 --!--


from manuskript.models import references as Ref
from manuskript.functions import mainWindow
from manuskript.enums import World
from PyQt5.QtWidgets import QTextEdit, QLineEdit
from manuskript.ui.highlighters.searchResultHighlighters.abstractSpecificSearchResultHighlighter import abstractSearchResultHighlighter


class worldSearchResultHighlighter(abstractSearchResultHighlighter):
    def __init__(self):
        super().__init__()

    def openView(self, searchResult):
        r = Ref.worldReference(searchResult.id())
        Ref.open(r)
        mainWindow().tabWorld.setEnabled(True)

    def retrieveWidget(self, searchResult):
        textEditMap = {
            World.name: (0, "txtWorldName", QLineEdit),
            World.description: (0, "txtWorldDescription", QTextEdit),
            World.passion: (1, "txtWorldPassion", QTextEdit),
            World.conflict: (1, "txtWorldConflict", QTextEdit),
        }

        tabIndex, widgetName, widgetClass = textEditMap[searchResult.column()]

        mainWindow().tabWorld.setCurrentIndex(tabIndex)
        return mainWindow().tabWorld.findChild(widgetClass, widgetName)



#!/usr/bin/env python
# --!-- coding: utf8 --!--
from PyQt5.QtCore import QModelIndex, QSortFilterProxyModel
from manuskript.enums import Character as C

class characterPOVModel(QSortFilterProxyModel):

    def __init__(self, sourceModel, parent=None):
        QSortFilterProxyModel.__init__(self, parent)

        self.setSourceModel(sourceModel)

        if sourceModel:
            sourceModel.dataChanged.connect(self.sourceDataChanged)

    def filterAcceptsRow(self, sourceRow, sourceParent):
        # Although I would prefer to reuse the existing characterModel.pov() method,
        # this is simpler to do, actually works and also more ideomatic Qt code.
        index = self.sourceModel().index(sourceRow, C.pov.value, sourceParent)
        value = self.sourceModel().data(index)
        return bool(value)

    def rowToSource(self, row):
        index = self.index(row, 0)
        sourceIndex = self.mapToSource(index)
        return sourceIndex.row()

    def sourceDataChanged(self, topLeft, bottomRight):
        self.invalidateFilter()

    ###############################################################################
    # CHARACTER QUERIES
    ###############################################################################

    def character(self, row):
        return self.sourceModel().character(self.rowToSource(row))

    def name(self, row):
        return self.sourceModel().name(self.rowToSource(row))

    def icon(self, row):
        return self.sourceModel().icon(self.rowToSource(row))

    def ID(self, row):
        return self.sourceModel().ID(self.rowToSource(row))

    def importance(self, row):
        return self.sourceModel().importance(self.rowToSource(row))

    def pov(self, row):
        return self.sourceModel().pov(self.rowToSource(row))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript/ui/listDialog_ui.ui'
#
# Created by: PyQt5 UI code generator 5.15.10
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_GenericListDialog(object):
    def setupUi(self, GenericListDialog):
        GenericListDialog.setObjectName("GenericListDialog")
        GenericListDialog.resize(451, 340)
        GenericListDialog.setModal(False)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(GenericListDialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(GenericListDialog)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.listWidget = QtWidgets.QListWidget(GenericListDialog)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout.addWidget(self.listWidget)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(GenericListDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout_2.addWidget(self.buttonBox)

        self.retranslateUi(GenericListDialog)
        self.buttonBox.accepted.connect(GenericListDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(GenericListDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(GenericListDialog)

    def retranslateUi(self, GenericListDialog):
        _translate = QtCore.QCoreApplication.translate
        GenericListDialog.setWindowTitle(_translate("GenericListDialog", "Title"))
        self.label.setText(_translate("GenericListDialog", "Text"))




class Signal():
    def __init__(self) -> None:
        self._methods = []

    def connect(self, func):
        self._methods.append(func)
    
    def disconnect(self, func):
        try:
            self._methods.remove(func)
        except ValueError:
            raise TypeError
        
    def disconnect(self):
        if len(self._methods) == 0:
            raise TypeError
        self._methods.pop()

    def fire(self, data):
        for m in self._methods:
            m(data)




#!/usr/bin/env python
# --!-- coding: utf8 --!--


class searchFilter:
    def __init__(self, label, enabled, modelColumns = None):
        if not isinstance(label, str):
            raise TypeError("label must be a str")

        if not isinstance(enabled, bool):
            raise TypeError("enabled must be a bool")

        if modelColumns is not None and (not isinstance(modelColumns, list)):
            raise TypeError("modelColumns must be a list or None")

        self._label = label
        self._enabled = enabled
        self._modelColumns = modelColumns
        if self._modelColumns is None:
            self._modelColumns = []

    def label(self):
        return self._label

    def enabled(self):
        return self._enabled

    def modelColumns(self):
        return self._modelColumns

    def setEnabled(self, enabled):
        self._enabled = enabled



#!/bin/python

import argparse
import os
from pathlib import Path
import subprocess

parser = argparse.ArgumentParser(
    description="Update all QM source translation binaries based on the source .ts files."
)
parser.add_argument(
    "--binpath", type=str, help="Path to the lrelease binary.", required=True
)
parser.add_argument(
    "--transpath",
    type=str,
    help="Path to the source translations directory.  Does not recurse.",
    default="./i18n/",
)


def cleanupFiles(binpath, transpath) -> None:
    # Catch for if the user doesn't pass in a path but we're still passing a None.
    src_files = Path(transpath) if transpath is not None else "./i18n/"
    src_files = [
        x for x in src_files.iterdir() if x.is_file() and str(x).endswith(".qm")
    ]

    for file in src_files:
        # lrelease.exe path_to_translation.qm
        subprocess.run([lrelease_path, str(file)])


# This permits us to import these steps discretely for a bigger build tool.
if __name__ == "__main__":
    arguments = parser.parse_args()

    # To permit calling as a normal function too ig.
    if arguments.binpath is not None:
        lrelease_path = Path(arguments.binpath)
    if arguments.transpath is not None:
        trans_path = Path(arguments.transpath)

    cleanupFiles(lrelease_path, trans_path)



#!/usr/bin/env python
# --!-- coding: utf8 --!--

"""Tests for functions"""

import re, os
from manuskript import functions as F

def test_wordCount():
    assert F.wordCount("In the beginning was the word.") == 6
    assert F.wordCount("") == 0

def test_convert():

    # toInt
    assert F.toInt("9") == 9
    assert F.toInt("a") == 0
    assert F.toInt("") == 0

    # toFloat
    assert F.toFloat("9.4") == 9.4
    assert F.toFloat("") == 0.

    # toString
    assert F.toString(None) == ""
    assert F.toString("None") == ""
    assert F.toString("Joy") == "Joy"

def test_several():

    from PyQt5.QtGui import QPainter, QPixmap, QIcon, QColor
    from PyQt5.QtCore import QRect

    # drawProgress
    px = QPixmap(10, 10)
    F.drawProgress(QPainter(px), QRect(0, 0, 100, 100), 0.5)

    # colorFromProgress
    a = F.colorFromProgress(0.1)
    b = F.colorFromProgress(0.5)
    c = F.colorFromProgress(1.0)
    d = F.colorFromProgress(1.5)
    assert a != b != c != d

    # iconColor & iconFromColor & iconFromColorString
    icon = F.iconFromColorString("#ff0000")
    assert F.iconColor(icon).name().lower() == "#ff0000"

    # themeIcon
    assert F.themeIcon("text") != None
    assert F.themeIcon("nonexistingname") != None

    # randomColor
    c1 = F.randomColor()
    c2 = F.randomColor(c1)
    assert c1.name() != c2.name()

    # mixColors
    c1 = QColor("#FFF")
    c2 = QColor("#000")
    assert F.mixColors(c1, c2).name() == "#7f7f7f"

    # colorifyPixmap
    assert F.colorifyPixmap(px, c1) != None

def test_outlineItemColors():

    from manuskript.models import outlineItem
    item = outlineItem(title="Test")

    r = F.outlineItemColors(item)
    for i in ["POV", "Label", "Progress", "Compile"]:
        assert i in r
    from PyQt5.QtGui import QColor
    assert r["Compile"].name(QColor.HexArgb) == "#00000000"

def test_paths():

    assert F.appPath() != None
    assert F.writablePath != None
    assert len(F.allPaths("suffix")) == 2
    assert F.tempFile("yop") != None
    f = F.findBackground("spacedreams.jpg")
    assert os.path.join("resources", "backgrounds", "spacedreams.jpg") in f
    assert len(F.customIcons()) > 1

def test_mainWindow():

    from PyQt5.QtWidgets import QWidget, QLCDNumber

    assert F.mainWindow() != None
    assert F.MW != None

    F.statusMessage("Test")
    F.printObjects()
    assert len(F.findWidgetsOfClass(QWidget)) > 0
    assert len(F.findWidgetsOfClass(QLCDNumber)) == 0


def test_search_noMatch():
    assert F.search(re.compile("text"), "foo") == []


def test_search_singleLine_fullMatch():
    assert F.search(re.compile("text"), "text") == [(0, 4, "<b>text</b>")]


def test_search_singleLine_start():
    assert F.search(re.compile("text"), "text is this") == [(0, 4, "<b>text</b> is this")]


def test_search_singleLine_end():
    assert F.search(re.compile("text"), "This is text") == [(8, 12, "This is <b>text</b>")]


def test_search_multipleLines_fullMatch():
    assert F.search(re.compile("text"), "This is\ntext\nOK") == [(8, 12, "[...] <b>text</b> [...]")]


def test_search_multipleLines_start():
    assert F.search(re.compile("text"), "This is\ntext oh yeah\nOK") == [(8, 12, "[...] <b>text</b> oh yeah [...]")]


def test_search_multipleLines_end():
    assert F.search(re.compile("text"), "This is\nsome text\nOK") == [(13, 17, "[...] some <b>text</b> [...]")]

def test_search_multipleLines_full():
    assert F.search(re.compile("text"), "This is\ntext\nOK") == [(8, 12, "[...] <b>text</b> [...]")]


def test_search_multiple_strMatches():
    assert F.search(re.compile("text"), "text, text and more text") == [
        (0, 4, "<b>text</b>, text and more text"),
        (6, 10, "text, <b>text</b> and more text"),
        (20, 24, "text, text and more <b>text</b>")
    ]


def test_search_multiple_strMatches_caseSensitive():
    assert F.search(re.compile("text"), "TeXt, TEXT and more text") == [(20, 24, "TeXt, TEXT and more <b>text</b>")]

    assert F.search(re.compile("text", re.IGNORECASE), "TeXt, TEXT and more text") == [
        (0, 4, "<b>TeXt</b>, TEXT and more text"),
        (6, 10, "TeXt, <b>TEXT</b> and more text"),
        (20, 24, "TeXt, TEXT and more <b>text</b>")
    ]


#!/usr/bin/env python
# --!-- coding: utf8 --!--

import subprocess
import pathlib
import importlib.util
import sys

# This is to produce an environment variable in GH Actions exclusively for use in CI/CD builds.


def writeVersionPlusHash() -> bool:
    """This function will permit us to rewrite
    the inline version.py file to contain the
    appropriate version-plus-hash-short to be
    shown in the about screen, version diagnostics
    and more.  This helps us better keep track
    of long-running issues in a relatively
    consise manner."""

    sha_short = (
        subprocess.run(
            "git rev-parse --short HEAD", shell=True, check=True, capture_output=True
        )
        .stdout.strip()
        .decode("utf-8")
    )  # UTF8 decode as it's a UTF8 situation.

    
    version_file = pathlib.Path("./manuskript/version.py").absolute()

    spec = importlib.util.spec_from_file_location(
        "manuskript.version", version_file.absolute()
    )
    temp = importlib.util.module_from_spec(spec)
    sys.modules["module.name"] = temp
    spec.loader.exec_module(temp)

    # used in final output.
    non_commit_version = temp.getVersion()

    # Used to rewrite the situation here.

    version_file_contents = open(str(version_file), 'r').readlines()

    for key, f in enumerate(version_file_contents):
        if ("__version__ = ") in f:
            version_file_contents[key] = f"__version__ = \"{non_commit_version}-{sha_short}\"\n"

    try:
        with open(str(version_file), 'w') as output_stub:
            output_stub.writelines(version_file_contents)
        return f"{non_commit_version}-{sha_short}"
    except:
        return None
        


#!/usr/bin/env python
# --!-- coding: utf8 --!--


class searchResultModel():
    def __init__(self, model_type, model_id, column, title, path, pos, context):
        self._type = model_type
        self._id = model_id
        self._column = column
        self._title = title
        self._path = path
        self._pos = pos
        self._context = context

    def type(self):
        return self._type

    def id(self):
        return self._id

    def column(self):
        return self._column

    def title(self):
        return self._title

    def path(self):
        return self._path

    def pos(self):
        return self._pos

    def context(self):
        return self._context

    def __repr__(self):
        return "(%s, %s, %s, %s, %s, %s, %s)" % (self._type, self._id, self._column, self._title, self._path, self._pos, self._context)

    def __eq__(self, other):
        return self.type() == other.type() and \
               self.id() == other.id() and \
               self.column == other.column and \
               self.pos() == other.pos() and \
               self.context == other.context



#!/usr/bin/env python
# --!-- coding: utf8 --!--
import re, textwrap

from PyQt5.Qt import QApplication
from PyQt5.QtCore import QTimer, QModelIndex, Qt, QEvent, pyqtSignal, QRegExp, QLocale, QPersistentModelIndex, QMutex
from PyQt5.QtGui import QTextBlockFormat, QTextCharFormat, QFont, QColor, QIcon, QMouseEvent, QTextCursor
from PyQt5.QtWidgets import QWidget, QTextEdit, qApp, QAction, QMenu, QToolTip

from manuskript import settings
from manuskript.enums import Outline, World, Character, Plot
from manuskript import functions as F
from manuskript.models import outlineModel, outlineItem
from manuskript.ui.highlighters import BasicHighlighter
from manuskript.ui import style as S
from manuskript.functions import Spellchecker
from manuskript.models.characterModel import Character, CharacterInfo


import logging
LOGGER = logging.getLogger(__name__)

# See implementation of QTextDocument::toPlainText()
PLAIN_TRANSLATION_TABLE = {0x2028: "\n", 0x2029: "\n", 0xfdd0: "\n", 0xfdd1: "\n"}

class textEditView(QTextEdit):

    def __init__(self, parent=None, index=None, html=None, spellcheck=None,
                 highlighting=False, dict="", autoResize=False):
        QTextEdit.__init__(self, parent)
        self._column = Outline.text
        self._index = None
        self._indexes = None
        self._model = None
        self._placeholderText = self.placeholderText()
        self._updating = QMutex()
        self._item = None
        self._highlighting = highlighting
        self._textFormat = "text"
        self.setAcceptRichText(False)
        # When setting up a theme, this becomes true.
        self._fromTheme = False
        self._themeData = None
        self._highlighterClass = BasicHighlighter

        if spellcheck == None:
            spellcheck = settings.spellcheck

        self.spellcheck = spellcheck
        self.currentDict = dict if dict else settings.dict
        self._defaultFontSize = qApp.font().pointSize()
        self.highlighter = None
        self.setAutoResize(autoResize)
        self._defaultBlockFormat = QTextBlockFormat()
        self._defaultCharFormat = QTextCharFormat()
        self.highlightWord = ""
        self.highligtCS = False
        self._dict = None
        self._tooltip = { 'depth' : 0, 'active' : 0 }

        # Submit text changed only after 500ms without modifications
        self.updateTimer = QTimer()
        self.updateTimer.destroyed.connect(self.cleanupTimer)
        self.updateTimer.setInterval(500)
        self.updateTimer.setSingleShot(True)
        self.updateTimer.timeout.connect(self.submit)

        self.updateTimer.stop()
        self.updateTimerConnection = self.document().contentsChanged.connect(self.updateTimer.start, F.AUC)

        self.setEnabled(False)

        if index:
            self.setCurrentModelIndex(index)

        elif html:
            self.document().setHtml(html)
            self.setReadOnly(True)

        # Spellchecking
        if self.spellcheck:
            self._dict = Spellchecker.getDictionary(self.currentDict)

        if not self._dict:
            self.spellcheck = False

        if self._highlighting and not self.highlighter:
            self.highlighter = self._highlighterClass(self)
            self.highlighter.setDefaultBlockFormat(self._defaultBlockFormat)
    
    def cleanupTimer(self):
        if self.updateTimer:
            self.disconnectDocument()
            self.updateTimer = None
    
    def __del__(self):
        self.cleanupTimer()

    def setModel(self, model):
        self._model = model
        try:
            self._model.dataChanged.connect(self.update, F.AUC)
        except TypeError:
            pass

    def setColumn(self, col):
        self._column = col

    def setHighlighting(self, val):
        self._highlighting = val

    def setDefaultBlockFormat(self, bf):
        self._defaultBlockFormat = bf
        if self.highlighter:
            self.highlighter.setDefaultBlockFormat(bf)

    def setCurrentModelIndex(self, index):
        self._indexes = None
        if index.isValid():
            self.setEnabled(True)
            if index.column() != self._column:
                index = index.sibling(index.row(), self._column)
            self._index = QPersistentModelIndex(index)

            self.setPlaceholderText(self._placeholderText)

            if not self._model:
                self.setModel(index.model())

            self.setupEditorForIndex(self._index)
            self.loadFontSettings()
            self.updateText()

        else:
            self._index = QModelIndex()

            self.setPlainText("")
            self.setEnabled(False)

    def currentIndex(self):
        """
        Getter function used to normalized views access with QAbstractItemViews.
        """
        if self._index:
            return self._index
        else:
            return QModelIndex()

    def getSelection(self):
        """
        Getter function used to normalized views access with QAbstractItemViews.
        """
        return [self.currentIndex()]

    def setCurrentModelIndexes(self, indexes):
        self._index = None
        self._indexes = []

        for i in indexes:
            if i.isValid():
                self.setEnabled(True)
                if i.column() != self._column:
                    i = i.sibling(i.row(), self._column)
                self._indexes.append(QModelIndex(i))

                if not self._model:
                    self.setModel(i.model())

        self.updateText()

    def setupEditorForIndex(self, index):
        # Setting highlighter
        if self._highlighting:
            self.highlighter = self._highlighterClass(self)
            self.highlighter.setDefaultBlockFormat(self._defaultBlockFormat)
            self.highlighter.updateColorScheme()

    def loadFontSettings(self):
        if self._fromTheme or \
            not self._index or \
                type(self._index.model()) != outlineModel or \
                self._column != Outline.text:
            return

        opt = settings.textEditor
        f = QFont()
        f.fromString(opt["font"])
        background = (opt["background"] if not opt["backgroundTransparent"]
                      else "transparent")
        foreground = opt["fontColor"]  # if not opt["backgroundTransparent"]
        #                               else S.text
        # self.setFont(f)
        self.setStyleSheet("""QTextEdit{{
            background: {bg};
            color: {foreground};
            font-family: {ff};
            font-size: {fs};
            margin: {mTB}px {mLR}px;
            {maxWidth}
            }}
            """.format(
            bg=background,
            foreground=foreground,
            ff=f.family(),
            fs="{}pt".format(str(f.pointSize())),
            mTB=opt["marginsTB"],
            mLR=opt["marginsLR"],
            maxWidth="max-width: {}px;".format(
                opt["maxWidth"]) if opt["maxWidth"] else "",
        )
        )
        self._defaultFontSize = f.pointSize()

        # We set the parent background to the editor's background in case
        # there are margins. We check that the parent class is a QWidget because
        # if textEditView is used in fullScreenEditor, then we don't want to
        # set the background.
        if self.parent().__class__ == QWidget:
            self.parent().setStyleSheet("""
                QWidget#{name}{{
                    background: {bg};
                }}""".format(
                # We style by name, otherwise all inheriting widgets get the same
                # colored background, for example context menu.
                name=self.parent().objectName(),
                bg=background,
            ))

        cf = QTextCharFormat()
        # cf.setFont(f)
        # cf.setForeground(QColor(opt["fontColor"]))

        self.setCursorWidth(opt["cursorWidth"])

        bf = QTextBlockFormat()
        bf.setLineHeight(opt["lineSpacing"], bf.ProportionalHeight)
        bf.setTextIndent(opt["tabWidth"] * 1 if opt["indent"] else 0)
        bf.setTopMargin(opt["spacingAbove"])
        bf.setBottomMargin(opt["spacingBelow"])
        bf.setAlignment(Qt.AlignLeft if opt["textAlignment"] == 0 else
                        Qt.AlignCenter if opt["textAlignment"] == 1 else
                        Qt.AlignRight if opt["textAlignment"] == 2 else
                        Qt.AlignJustify)

        self._defaultCharFormat = cf
        self._defaultBlockFormat = bf

        if self.highlighter:
            self.highlighter.updateColorScheme()
            self.highlighter.setMisspelledColor(QColor(opt["misspelled"]))
            self.highlighter.setDefaultCharFormat(self._defaultCharFormat)
            self.highlighter.setDefaultBlockFormat(self._defaultBlockFormat)

    def update(self, topLeft, bottomRight):
        update = False

        if self._index and self._index.isValid():
            if topLeft.parent() != self._index.parent():
                return

                # LOGGER.debug("Model changed: ({}:{}), ({}:{}/{}), ({}:{}) for {} of {}".format(
                # topLeft.row(), topLeft.column(),
                # self._index.row(), self._index.row(), self._column,
                # bottomRight.row(), bottomRight.column(),
                # self.objectName(), self.parent().objectName()))

            if topLeft.row() <= self._index.row() <= bottomRight.row():
                if topLeft.column() <= self._column <= bottomRight.column():
                    update = True

        elif self._indexes:
            for i in self._indexes:
                if topLeft.row() <= i.row() <= bottomRight.row():
                    update = True

        if update:
            self.updateText()

    def disconnectDocument(self):
        if not self.updateTimerConnection:
            return

        try:
            self.document().contentsChanged.disconnect(self.updateTimerConnection)
            self.updateTimerConnection = None
        except:
            pass

    def reconnectDocument(self):
        if not self.updateTimer or self.updateTimerConnection:
            return

        self.updateTimerConnection = self.document().contentsChanged.connect(self.updateTimer.start, F.AUC)

    def toIdealText(self):
        """QTextDocument::toPlainText() replaces NBSP with spaces, which we don't want.
        QTextDocument::toRawText() replaces nothing, but that leaves fancy paragraph and line separators that users would likely complain about.
        This reimplements toPlainText(), except without the NBSP destruction."""
        return self.document().toRawText().translate(PLAIN_TRANSLATION_TABLE)
    toPlainText = toIdealText

    def updateText(self):
        self._updating.lock()

        # LOGGER.debug("Updating %s", self.objectName())
        if self._index:
            self.disconnectDocument()
            if self.toIdealText() != F.toString(self._index.data()):
                # LOGGER.debug("    Updating plaintext")
                self.document().setPlainText(F.toString(self._index.data()))
            self.reconnectDocument()

        elif self._indexes:
            self.disconnectDocument()
            t = []
            same = True
            for i in self._indexes:
                item = i.internalPointer()
                t.append(F.toString(item.data(self._column)))

            for t2 in t[1:]:
                if t2 != t[0]:
                    same = False
                    break

            if same:
                self.document().setPlainText(t[0])
            else:
                self.document().setPlainText("")

                if not self._placeholderText:
                    self._placeholderText = self.placeholderText()

                self.setPlaceholderText(self.tr("Various"))
            self.reconnectDocument()

        self._updating.unlock()

    def submit(self):
        if self.updateTimer:
            self.updateTimer.stop()

        self._updating.lock()
        text = self.toIdealText()
        self._updating.unlock()

        # LOGGER.debug("Submitting %s", self.objectName())
        if self._index and self._index.isValid():
            # item = self._index.internalPointer()
            if text != self._index.data():
                # LOGGER.debug("    Submitting plain text")
                self._model.setData(QModelIndex(self._index), text)

        elif self._indexes:
            for i in self._indexes:
                item = i.internalPointer()
                if text != F.toString(item.data(self._column)):
                    LOGGER.debug("Submitting many indexes")
                    self._model.setData(i, text)

    def keyPressEvent(self, event):
        if event.key() == Qt.Key_V and event.modifiers() & Qt.ControlModifier:
            text = QApplication.clipboard().text()
            self.insertPlainText(text)
        else:
            QTextEdit.keyPressEvent(self, event)

        if event.key() == Qt.Key_Space:
            self.submit()

    # -----------------------------------------------------------------------------------------------------
    # Resize stuff

    def resizeEvent(self, e):
        QTextEdit.resizeEvent(self, e)
        if self._autoResize:
            self.sizeChange()

    def sizeChange(self):
        opt = settings.textEditor
        docHeight = self.document().size().height() + 2 * opt["marginsTB"]
        if self.heightMin <= docHeight <= self.heightMax:
            self.setMinimumHeight(int(docHeight))

    def setAutoResize(self, val):
        self._autoResize = val
        if self._autoResize:
            self.document().contentsChanged.connect(self.sizeChange)
            self.heightMin = 0
            self.heightMax = 65000
            self.sizeChange()

        ###############################################################################
        # SPELLCHECKING
        ###############################################################################
        # Based on http://john.nachtimwald.com/2009/08/22/qplaintextedit-with-in-line-spell-check/

    def setDict(self, d):
        self.currentDict = d
        if d:
            self._dict = Spellchecker.getDictionary(d)
        if self.highlighter:
            self.highlighter.rehighlight()

    def toggleSpellcheck(self, v):
        self.spellcheck = v
        if self.spellcheck and not self._dict:
            self._dict = Spellchecker.getDictionary(self.currentDict)

        if not self._dict:
            self.spellcheck = False

        if self.highlighter:
            self.highlighter.rehighlight()
        else:
            self.spellcheck = False

    def mousePressEvent(self, event):
        if event.button() == Qt.RightButton:
            # Rewrite the mouse event to a left button event so the cursor is
            # moved to the location of the pointer.
            event = QMouseEvent(QEvent.MouseButtonPress, event.pos(),
                                Qt.LeftButton, Qt.LeftButton, Qt.NoModifier)
        QTextEdit.mousePressEvent(self, event)

    def beginTooltipMoveEvent(self):
        self._tooltip['depth'] += 1

    def endTooltipMoveEvent(self):
        self._tooltip['depth'] -= 1

    def showTooltip(self, pos, text):
        QToolTip.showText(pos, text)
        self._tooltip['active'] = self._tooltip['depth']

    def hideTooltip(self):
        if self._tooltip['active'] == self._tooltip['depth']:
            QToolTip.hideText()

    def mouseMoveEvent(self, event):
        """
        When mouse moves, we show tooltip when appropriate.
        """
        self.beginTooltipMoveEvent()
        QTextEdit.mouseMoveEvent(self, event)
        self.endTooltipMoveEvent()

        match = None

        # Check if the selected word has any suggestions for correction
        if self.spellcheck and self._dict:
            cursor = self.cursorForPosition(event.pos())

            # Searches for correlating/overlapping matches
            suggestions = self._dict.findSuggestions(self.toPlainText(), cursor.selectionStart(), cursor.selectionEnd())

            if len(suggestions) > 0:
                # I think it should focus on one type of error at a time.
                match = suggestions[0]

        if match:
            # Wrap the message into a fitting width
            msg_lines = textwrap.wrap(match.msg, 48)

            self.showTooltip(event.globalPos(), "\n".join(msg_lines))
        else:
            self.hideTooltip()

    def wheelEvent(self, event):
        """
        We catch wheelEvent if key modifier is CTRL to change font size.
        Note: this should be in a class specific for main textEditView (#TODO).
        """
        if event.modifiers() & Qt.ControlModifier:
            # Get the wheel angle.
            d = event.angleDelta().y() / 120

            # Update settings
            f = QFont()
            f.fromString(settings.textEditor["font"])
            f.setPointSizeF(f.pointSizeF() + d)
            settings.textEditor["font"] = f.toString()

            # Update font to all textEditView. Drastically.
            for w in F.mainWindow().findChildren(textEditView, QRegExp(".*")):
                w.loadFontSettings()

            # We tell the world that we accepted this event
            event.accept()
            return

        QTextEdit.wheelEvent(self, event)

    class SpellAction(QAction):
        """A special QAction that returns the text in a signal. Used for spellcheck."""

        correct = pyqtSignal(str)

        def __init__(self, *args):
            QAction.__init__(self, *args)

            self.triggered.connect(lambda x: self.correct.emit(
                str(self.text())))

    def contextMenuEvent(self, event):
        # Based on http://john.nachtimwald.com/2009/08/22/qplaintextedit-with-in-line-spell-check/
        popup_menu = self.createStandardContextMenu()
        popup_menu.exec_(event.globalPos())

    def newCharacter(self):
        text = self.sender().data()
        LOGGER.debug(f'New character: {text}')
        # switch to character page
        mw = F.mainWindow()
        mw.tabMain.setCurrentIndex(mw.TabPersos)
        # add character
        c = mw.mdlCharacter.addCharacter(name=text)
        # switch to character
        item = mw.lstCharacters.getItemByID(c.ID())
        mw.lstCharacters.setCurrentItem(item)

    def newPlotItem(self):
        text = self.sender().data()
        LOGGER.debug(f'New plot item: {text}')
        # switch to plot page
        mw = F.mainWindow()
        mw.tabMain.setCurrentIndex(mw.TabPlots)
        # add character
        p, ID = mw.mdlPlots.addPlot(text)
        # switch to character
        plotIndex = mw.mdlPlots.getIndexFromID(ID.text())
        # segfaults for some reason
        # mw.lstSubPlots.setCurrentIndex(plotIndex)

    def newWorldItem(self):
        text = self.sender().data()
        LOGGER.debug(f'New world item: {text}')
        mw = F.mainWindow()
        mw.tabMain.setCurrentIndex(mw.TabWorld)
        item = mw.mdlWorld.addItem(title=text)
        mw.treeWorld.setCurrentIndex(
            mw.mdlWorld.indexFromItem(item))


    def appendContextMenuEntriesForWord(self, popup_menu, selectedWord):
        # add "new <something>" buttons at end
        if selectedWord != None:
            # new character
            charAction = QAction(self.tr("&New Character"), popup_menu)
            charAction.setIcon(F.themeIcon("characters"))
            charAction.triggered.connect(self.newCharacter)
            charAction.setData(selectedWord)
            popup_menu.insertAction(None, charAction)

            # new plot item
            plotAction = QAction(self.tr("&New Plot Item"), popup_menu)
            plotAction.setIcon(F.themeIcon("plots"))
            plotAction.triggered.connect(self.newPlotItem)
            plotAction.setData(selectedWord)
            popup_menu.insertAction(None, plotAction)

            # new world item
            worldAction = QAction(self.tr("&New World Item"), popup_menu)
            worldAction.setIcon(F.themeIcon("world"))
            worldAction.triggered.connect(self.newWorldItem)
            worldAction.setData(selectedWord)
            popup_menu.insertAction(None, worldAction)

        return popup_menu

    def createStandardContextMenu(self):
        popup_menu = QTextEdit.createStandardContextMenu(self)

        cursor = self.textCursor()
        selectedWord = cursor.selectedText() if cursor.hasSelection() else None

        if not self.spellcheck:
            return self.appendContextMenuEntriesForWord(popup_menu, selectedWord)

        suggestions = []

        # Check for any suggestions for corrections at the cursors position
        if self._dict is not None:
            text = self.toPlainText()

            suggestions = self._dict.findSuggestions(text, cursor.selectionStart(), cursor.selectionEnd())

            # Select the word under the cursor if necessary.
            # But only if there is no selection (otherwise it's impossible to select more text to copy/cut)
            if not cursor.hasSelection() and len(suggestions) == 0:
                old_position = cursor.position()

                cursor.select(QTextCursor.WordUnderCursor)
                self.setTextCursor(cursor)

                if cursor.hasSelection():
                    selectedWord = cursor.selectedText()

                    # Check if the selected word is misspelled and offer spelling
                    # suggestions if it is.
                    suggestions = self._dict.findSuggestions(text, cursor.selectionStart(), cursor.selectionEnd())

                if len(suggestions) == 0:
                    cursor.clearSelection()
                    cursor.setPosition(old_position, QTextCursor.MoveAnchor)
                    self.setTextCursor(cursor)

                    selectedWord = None

        popup_menu = self.appendContextMenuEntriesForWord(popup_menu, selectedWord)

        if len(suggestions) > 0 or selectedWord is not None:
            valid = len(suggestions) == 0

            if not valid:
                # I think it should focus on one type of error at a time.
                match = suggestions[0]

                popup_menu.insertSeparator(popup_menu.actions()[0])

                if match.locqualityissuetype == 'misspelling':
                    spell_menu = QMenu(self.tr('Spelling Suggestions'), self)
                    spell_menu.setIcon(F.themeIcon("spelling"))

                    if match.end > match.start and selectedWord is None:
                        # Select the actual area of the match
                        cursor = self.textCursor()
                        cursor.setPosition(match.start, QTextCursor.MoveAnchor);
                        cursor.setPosition(match.end, QTextCursor.KeepAnchor);
                        self.setTextCursor(cursor)

                        selectedWord = cursor.selectedText()

                    if match.replacements:
                        for word in match.replacements:
                            action = self.SpellAction(word, spell_menu)
                            action.correct.connect(self.correctWord)
                            spell_menu.addAction(action)

                    # Adds: add to dictionary
                    addAction = QAction(self.tr("&Add to dictionary"), popup_menu)
                    addAction.setIcon(QIcon.fromTheme("list-add"))
                    addAction.triggered.connect(self.addWordToDict)
                    addAction.setData(selectedWord)

                    popup_menu.insertAction(popup_menu.actions()[0], addAction)

                    # Only add the spelling suggests to the menu if there are
                    # suggestions.
                    if match.replacements and len(match.replacements) > 0:
                        # Adds: suggestions
                        popup_menu.insertMenu(popup_menu.actions()[0], spell_menu)
                else:
                    correct_menu = None
                    correct_action = None

                    if len(match.replacements) > 0 and match.end > match.start:
                        # Select the actual area of the match
                        cursor = self.textCursor()
                        cursor.setPosition(match.start, QTextCursor.MoveAnchor);
                        cursor.setPosition(match.end, QTextCursor.KeepAnchor);
                        self.setTextCursor(cursor)

                        if len(match.replacements) > 0:
                            correct_menu = QMenu(self.tr('&Correction Suggestions'), self)
                            correct_menu.setIcon(F.themeIcon("spelling"))

                            for word in match.replacements:
                                action = self.SpellAction(word, correct_menu)
                                action.correct.connect(self.correctWord)
                                correct_menu.addAction(action)

                    if correct_menu is None:
                        correct_action = QAction(self.tr('&Correction Suggestion'), popup_menu)
                        correct_action.setIcon(F.themeIcon("spelling"))
                        correct_action.setEnabled(False)

                    # Wrap the message into a fitting width
                    msg_lines = textwrap.wrap(match.msg, 48)

                    # Insert the lines of the message backwards
                    for i in range(0, len(msg_lines)):
                        popup_menu.insertSection(popup_menu.actions()[0], msg_lines[len(msg_lines) - (i + 1)])

                    if correct_menu is None:
                        popup_menu.insertAction(popup_menu.actions()[0], correct_action)
                    else:
                        popup_menu.insertMenu(popup_menu.actions()[0], correct_menu)

            # If word was added to custom dict, give the possibility to remove it
            elif self._dict.isCustomWord(selectedWord):
                popup_menu.insertSeparator(popup_menu.actions()[0])
                # Adds: remove from dictionary
                rmAction = QAction(
                    self.tr("&Remove from custom dictionary"), popup_menu)
                rmAction.setIcon(QIcon.fromTheme("list-remove"))
                rmAction.triggered.connect(self.rmWordFromDict)
                rmAction.setData(selectedWord)
                popup_menu.insertAction(popup_menu.actions()[0], rmAction)

        return popup_menu

    def correctWord(self, word):
        """
        Replaces the selected text with word.
        """
        cursor = self.textCursor()
        cursor.beginEditBlock()

        cursor.removeSelectedText()
        cursor.insertText(word)

        cursor.endEditBlock()

    def addWordToDict(self):
        word = self.sender().data()
        self._dict.addWord(word)
        self.highlighter.rehighlight()

    def rmWordFromDict(self):
        word = self.sender().data()
        self._dict.removeWord(word)
        self.highlighter.rehighlight()

    ###############################################################################
    # FORMATTING
    ###############################################################################

    def focusOutEvent(self, event):
        """Submit changes just before focusing out."""
        QTextEdit.focusOutEvent(self, event)
        self.submit()

    ###############################################################################
    # KEYBOARD SHORTCUTS
    ###############################################################################

    def callMainTreeView(self, functionName):
        """
        The tree view in main window must have same index as the text
        edit that has focus. So we can pass it the call for documents
        edits like: duplicate, move up, etc.
        """
        if self._index and self._column == Outline.text:
            function = getattr(F.mainWindow().treeRedacOutline, functionName)
            function()

    def rename(self): self.callMainTreeView("rename")
    def duplicate(self): self.callMainTreeView("duplicate")
    def moveUp(self): self.callMainTreeView("moveUp")
    def moveDown(self): self.callMainTreeView("moveDown")
    def delete(self): self.callMainTreeView("delete")



#!/usr/bin/env python
# --!-- coding: utf8 --!--
from PyQt5.QtWidgets import QTextEdit, qApp

from manuskript.exporter.basic import basicExporter, basicFormat
from manuskript.exporter.manuskript.HTML import HTML
from manuskript.exporter.manuskript.markdown import markdown
from manuskript.exporter.manuskript.plainText import plainText
from manuskript.functions import appPath, safeTranslate

import os


class manuskriptExporter(basicExporter):

    name = "Manuskript"
    description = safeTranslate(qApp, "Export", "Default exporter, provides basic formats used by other exporters.")
    exportTo = [
        plainText(),
        markdown(),
        HTML(),
        basicFormat("OPML", icon="text-x-opml+xml")
    ]
    icon = appPath(os.path.join("icons", "Manuskript", "icon-256px.png"))

    @classmethod
    def isValid(cls):
        return True


#!/usr/bin/env python
# --!-- coding: utf8 --!--

from manuskript.functions import mainWindow
from manuskript.enums import  FlatData
from PyQt5.QtWidgets import QTextEdit, QLineEdit
from manuskript.ui.highlighters.searchResultHighlighters.abstractSpecificSearchResultHighlighter import abstractSearchResultHighlighter


class flatDataSearchResultHighlighter(abstractSearchResultHighlighter):
    def __init__(self):
        super().__init__()

    def openView(self, searchResult):
        mainWindow().tabMain.setCurrentIndex(mainWindow().TabSummary)

    def retrieveWidget(self, searchResult):
        editors = {
            FlatData.summarySituation: (0, "txtSummarySituation", QLineEdit, mainWindow()),
            FlatData.summarySentence: (0, "txtSummarySentence", QTextEdit, mainWindow().tabSummary),
            FlatData.summaryPara: (1, "txtSummaryPara", QTextEdit, mainWindow().tabSummary),
            FlatData.summaryPage: (2, "txtSummaryPage", QTextEdit, mainWindow().tabSummary),
            FlatData.summaryFull: (3, "txtSummaryFull", QTextEdit, mainWindow().tabSummary)
        }

        stackIndex, editorName, editorClass, rootWidget = editors[searchResult.column()]

        mainWindow().tabSummary.setCurrentIndex(stackIndex)
        return rootWidget.findChild(editorClass, editorName)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript/ui/characterInfoDialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_characterInfoDialog(object):
    def setupUi(self, characterInfoDialog):
        characterInfoDialog.setObjectName("characterInfoDialog")
        characterInfoDialog.setWindowModality(QtCore.Qt.WindowModal)
        characterInfoDialog.resize(481, 148)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(characterInfoDialog.sizePolicy().hasHeightForWidth())
        characterInfoDialog.setSizePolicy(sizePolicy)
        characterInfoDialog.setMinimumSize(QtCore.QSize(481, 148))
        characterInfoDialog.setMaximumSize(QtCore.QSize(481, 148))
        characterInfoDialog.setTabletTracking(False)
        characterInfoDialog.setFocusPolicy(QtCore.Qt.NoFocus)
        characterInfoDialog.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedKingdom))
        self.gridLayout = QtWidgets.QGridLayout(characterInfoDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(characterInfoDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.valueLineEdit = QtWidgets.QLineEdit(characterInfoDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.valueLineEdit.sizePolicy().hasHeightForWidth())
        self.valueLineEdit.setSizePolicy(sizePolicy)
        self.valueLineEdit.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.valueLineEdit.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.valueLineEdit.setObjectName("valueLineEdit")
        self.horizontalLayout_2.addWidget(self.valueLineEdit)
        self.gridLayout.addLayout(self.horizontalLayout_2, 3, 0, 1, 1)
        self.horizontalLayout_1 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_1.setObjectName("horizontalLayout_1")
        self.label = QtWidgets.QLabel(characterInfoDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.horizontalLayout_1.addWidget(self.label)
        self.descriptionLineEdit = QtWidgets.QLineEdit(characterInfoDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.descriptionLineEdit.sizePolicy().hasHeightForWidth())
        self.descriptionLineEdit.setSizePolicy(sizePolicy)
        self.descriptionLineEdit.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.descriptionLineEdit.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.descriptionLineEdit.setObjectName("descriptionLineEdit")
        self.horizontalLayout_1.addWidget(self.descriptionLineEdit)
        self.gridLayout.addLayout(self.horizontalLayout_1, 2, 0, 1, 1)
        self.buttonBox = QtWidgets.QDialogButtonBox(characterInfoDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.buttonBox.sizePolicy().hasHeightForWidth())
        self.buttonBox.setSizePolicy(sizePolicy)
        self.buttonBox.setFocusPolicy(QtCore.Qt.StrongFocus)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 4, 0, 1, 1)

        self.retranslateUi(characterInfoDialog)
        self.buttonBox.accepted.connect(characterInfoDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(characterInfoDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(characterInfoDialog)
        characterInfoDialog.setTabOrder(self.descriptionLineEdit, self.valueLineEdit)

    def retranslateUi(self, characterInfoDialog):
        _translate = QtCore.QCoreApplication.translate
        characterInfoDialog.setWindowTitle(_translate("characterInfoDialog", "Add Character Info"))
        self.label_2.setText(_translate("characterInfoDialog", "Value:"))
        self.label.setText(_translate("characterInfoDialog", "Name:"))



#!/usr/bin/env python
# --!-- coding: utf8 --!--

"""Test ParseMMDFile function."""

from collections import OrderedDict
from manuskript.load_save.version_1 import parseMMDFile


BASE = "title:          TheExampleNovel\n"
BASE += "ID:             42\n"
BASE += "type:           folder\n"
TEXT = BASE + '\n'
TEXT_WITH_NONE_AT_START = "None:           hello\n" + TEXT
TEXT_WITH_NONE_AT_END = BASE + "None:           hello\n\n"
TEXT_WITH_HANGING_SPACE = BASE + "     "
CONTENT = "Once upon a time, there was a dog"
TEXT_WITH_CONTENT = TEXT + CONTENT


def test_empty_string():
    """An empty string given to parseMMDFile."""
    result = parseMMDFile("")
    assert result == ([], '')


def test_text():
    """A result as a list of tuples"""
    result = parseMMDFile(TEXT)
    assert result == ([
        ('title', 'TheExampleNovel'),
        ('ID', '42'),
        ('type', 'folder')
    ], '')


def test_text_asdict():
    """A result as an OrderedDict."""
    result = parseMMDFile(TEXT, True)
    assert result == (OrderedDict([
        ('title', 'TheExampleNovel'),
        ('ID', '42'),
        ('type', 'folder')]
    ), '')


def test_text_with_none_at_start():
    """If the description is None, replace with an empty string."""
    result = parseMMDFile(TEXT_WITH_NONE_AT_START)
    assert result == ([
        ('', 'hello'),
        ('title', 'TheExampleNovel'),
        ('ID', '42'),
        ('type', 'folder')
    ], '')


def test_text_wth_none_at_end():
    """If the last description is None, replace with an empty string."""
    result = parseMMDFile(TEXT_WITH_NONE_AT_END)
    assert result == ([
        ('title', 'TheExampleNovel'),
        ('ID', '42'),
        ('type', 'folder'),
        ('', 'hello')
    ], '')


def test_text_hanging_space():
    """Hanging space invalidates the line."""
    result = parseMMDFile(TEXT_WITH_HANGING_SPACE)
    assert result == ([
        ('title', 'TheExampleNovel'),
        ('ID', '42')
    ], '')


def test_text_with_content():
    """The scene has text."""
    result = parseMMDFile(TEXT_WITH_CONTENT)
    assert result[1] == CONTENT



from manuskript.functions.history.NavigatedEvent import NavigatedEvent
from manuskript.functions.history.Signal import Signal


class History():
    def __init__(self) -> None:
        self._entries = []
        self._position = 0
        self.navigated = Signal()
        self._navigating = False

    def next(self, entry):
        if self._navigating:
            return

        while self._position < len(self._entries) - 1:
            self._entries.pop()
        
        self._entries.append(entry)
        self._position = len(self._entries) - 1
        self._navigating = True
        self.navigated.fire(NavigatedEvent(self._position, len(self._entries), entry))
        self._navigating = False

    
    def replace(self, entry):
        if self._navigating:
            return
    
        while self._position < len(self._entries):
            self._entries.pop()

        self._entries.append(entry)
        self._position = len(self._entries) - 1
        self._navigating = True
        self.navigated.fire(NavigatedEvent(self._position, len(self._entries), entry))
        self._navigating = False

    def forward(self):
        if self._position < len(self._entries) - 1:
            self._position += 1
            self._navigating = True
            self.navigated.fire(NavigatedEvent(self._position, len(self._entries), self._entries[self._position]))
            self._navigating = False

    def back(self):
        if self._position > 0:
            self._position -= 1
            self._navigating = True
            self.navigated.fire(NavigatedEvent(self._position, len(self._entries), self._entries[self._position]))
            self._navigating = False

    def reset(self):
        self._entries.clear()
        self._position = 0
        self.navigated.fire(NavigatedEvent(self._position, len(self._entries), None))



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'manuskript/ui/tools/targets_ui.ui'
#
# Created by: PyQt5 UI code generator 5.15.11
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_targets(object):
    def setupUi(self, targets):
        targets.setObjectName("targets")
        targets.resize(322, 216)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap("manuskript/ui/tools/../../../../../../../../../../home/icons/Manuskript/icon-64px.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        targets.setWindowIcon(icon)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(targets)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.line = QtWidgets.QFrame(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line.sizePolicy().hasHeightForWidth())
        self.line.setSizePolicy(sizePolicy)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.horizontalLayout.addWidget(self.line)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        self.draft_progress_bar = QtWidgets.QProgressBar(targets)
        self.draft_progress_bar.setProperty("value", 0)
        self.draft_progress_bar.setObjectName("draft_progress_bar")
        self.verticalLayout_2.addWidget(self.draft_progress_bar)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.draft_wc_label = QtWidgets.QLabel(targets)
        self.draft_wc_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.draft_wc_label.setObjectName("draft_wc_label")
        self.horizontalLayout_4.addWidget(self.draft_wc_label)
        self.draft_of_label = QtWidgets.QLabel(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.draft_of_label.sizePolicy().hasHeightForWidth())
        self.draft_of_label.setSizePolicy(sizePolicy)
        self.draft_of_label.setObjectName("draft_of_label")
        self.horizontalLayout_4.addWidget(self.draft_of_label)
        self.draft_goal_label = QtWidgets.QLabel(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.draft_goal_label.sizePolicy().hasHeightForWidth())
        self.draft_goal_label.setSizePolicy(sizePolicy)
        self.draft_goal_label.setObjectName("draft_goal_label")
        self.horizontalLayout_4.addWidget(self.draft_goal_label)
        self.label_8 = QtWidgets.QLabel(targets)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_4.addWidget(self.label_8)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_2 = QtWidgets.QLabel(targets)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_2.addWidget(self.label_2)
        self.line_2 = QtWidgets.QFrame(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.line_2.sizePolicy().hasHeightForWidth())
        self.line_2.setSizePolicy(sizePolicy)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout_2.addWidget(self.line_2)
        self.session_reset = QtWidgets.QPushButton(targets)
        self.session_reset.setObjectName("session_reset")
        self.horizontalLayout_2.addWidget(self.session_reset)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.session_progress_bar = QtWidgets.QProgressBar(targets)
        self.session_progress_bar.setProperty("value", 0)
        self.session_progress_bar.setObjectName("session_progress_bar")
        self.verticalLayout_2.addWidget(self.session_progress_bar)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.session_wc_label = QtWidgets.QLabel(targets)
        self.session_wc_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.session_wc_label.setObjectName("session_wc_label")
        self.horizontalLayout_3.addWidget(self.session_wc_label)
        self.label_3 = QtWidgets.QLabel(targets)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.session_target = QtWidgets.QDoubleSpinBox(targets)
        self.session_target.setDecimals(0)
        self.session_target.setMaximum(99999999999999.0)
        self.session_target.setSingleStep(50.0)
        self.session_target.setProperty("value", 500.0)
        self.session_target.setObjectName("session_target")
        self.horizontalLayout_3.addWidget(self.session_target)
        self.label_5 = QtWidgets.QLabel(targets)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_3.addWidget(self.label_5)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)

        self.retranslateUi(targets)
        QtCore.QMetaObject.connectSlotsByName(targets)

    def retranslateUi(self, targets):
        _translate = QtCore.QCoreApplication.translate
        targets.setWindowTitle(_translate("targets", "Project Targets"))
        self.label.setText(_translate("targets", "Draft Target"))
        self.draft_wc_label.setText(_translate("targets", "0"))
        self.draft_of_label.setText(_translate("targets", "of"))
        self.draft_goal_label.setText(_translate("targets", "10,000"))
        self.label_8.setText(_translate("targets", "Words"))
        self.label_2.setText(_translate("targets", "Session Target"))
        self.session_reset.setText(_translate("targets", "Reset"))
        self.session_wc_label.setText(_translate("targets", "0"))
        self.label_3.setText(_translate("targets", "of"))
        self.label_5.setText(_translate("targets", "words"))



#!/usr/bin/env python
# --!-- coding: utf8 --!--

from manuskript.ui.highlighters.searchResultHighlighters.abstractSearchResultHighlighter import abstractSearchResultHighlighter
from manuskript.ui.highlighters.searchResultHighlighters.characterSearchResultHighlighter import characterSearchResultHighlighter
from manuskript.ui.highlighters.searchResultHighlighters.flatDataSearchResultHighlighter import flatDataSearchResultHighlighter
from manuskript.ui.highlighters.searchResultHighlighters.outlineSearchResultHighlighter import outlineSearchResultHighlighter
from manuskript.ui.highlighters.searchResultHighlighters.worldSearchResultHighlighter import worldSearchResultHighlighter
from manuskript.ui.highlighters.searchResultHighlighters.plotSearchResultHighlighter import plotSearchResultHighlighter
from manuskript.ui.highlighters.searchResultHighlighters.plotStepSearchResultHighlighter import plotStepSearchResultHighlighter
from manuskript.enums import Model


class searchResultHighlighter(abstractSearchResultHighlighter):
    def __init__(self):
        super().__init__()

    def highlightSearchResult(self, searchResult):
        if searchResult.type() == Model.Character:
            highlighter = characterSearchResultHighlighter()
        elif searchResult.type() == Model.FlatData:
            highlighter = flatDataSearchResultHighlighter()
        elif searchResult.type() == Model.Outline:
            highlighter = outlineSearchResultHighlighter()
        elif searchResult.type() == Model.World:
            highlighter = worldSearchResultHighlighter()
        elif searchResult.type() == Model.Plot:
            highlighter = plotSearchResultHighlighter()
        elif searchResult.type() == Model.PlotStep:
            highlighter = plotStepSearchResultHighlighter()
        else:
            raise NotImplementedError

        highlighter.highlightSearchResult(searchResult)



from PyQt5.QtWidgets import QDialog
from manuskript.ui.listDialog_ui import Ui_GenericListDialog


class ListDialog(QDialog, Ui_GenericListDialog):
    def __init__(self, parent=None):
        QDialog.__init__(self, parent)
        self.setupUi(self)

    def accept(self):
        self.hide()
        self.close()

    def reject(self):
        self.hide()
        self.close()



#!/usr/bin/env python
# --!-- coding: utf8 --!--
import re
from PyQt5.QtGui import QFont, QTextCharFormat
from PyQt5.QtWidgets import QPlainTextEdit, qApp, QFrame, QFileDialog, QMessageBox

from manuskript.exporter.basic import basicFormat
from manuskript.functions import mainWindow, getSaveFileNameWithSuffix, safeTranslate
from manuskript.models import outlineItem
from manuskript.ui.exporters.manuskript.plainTextSettings import exporterSettings
import codecs

import logging
LOGGER = logging.getLogger(__name__)

class plainText(basicFormat):
    name = safeTranslate(qApp, "Export", "Plain text")
    description = safeTranslate(qApp, "Export", """Simplest export to plain text. Allows you to use your own markup not understood
                  by Manuskript, for example <a href='www.fountain.io'>Fountain</a>.""")
    implemented = True
    requires = {
        "Settings": True,
        "Preview": True,
    }
    icon = "text-plain"

    # Default settings used in self.getExportFilename. For easy subclassing when exporting plaintext.
    exportVarName = "lastPlainText"
    exportFilter = "Text files (*.txt);; Any files (*)"
    exportDefaultSuffix = ".txt"  # qt ignores the period, but it is clearer in our code to have it

    def __init__(self):
        pass

    def settingsWidget(self):
        w = exporterSettings(self)
        w.loadSettings()
        return w

    def previewWidget(self):
        w = QPlainTextEdit()
        w.setFrameShape(QFrame.NoFrame)
        w.setReadOnly(True)
        return w

    def output(self, settingsWidget):
        settings = settingsWidget.getSettings()
        try:
            return self.concatenate(mainWindow().mdlOutline.rootItem, settings)
        except re.error as e:
            QMessageBox.warning(mainWindow().dialog, safeTranslate(qApp, "Export", "Error"),
                                safeTranslate(qApp, "Export", "Could not process regular expression: \n{}").format(str(e)))
            return ""

    def getExportFilename(self, settingsWidget, varName=None, filter=None):

        if varName == None:
            varName = self.exportVarName

        if filter == None:
            filter = self.exportFilter

        settings = settingsWidget.getSettings()

        s = settings.get("Output", {})
        if varName in s:
            filename = s[varName]
        else:
            filename = ""

        filename, filter = getSaveFileNameWithSuffix(settingsWidget.parent(),
                                                     caption=safeTranslate(qApp, "Export", "Choose output file…"),
                                                     filter=filter,
                                                     directory=filename,
                                                     defaultSuffix=self.exportDefaultSuffix)

        if filename:
            s[varName] = filename
            settingsWidget.settings["Output"] = s

            # Save settings
            settingsWidget.writeSettings()

        return filename

    def export(self, settingsWidget):
        settings = settingsWidget.getSettings()

        filename = self.getExportFilename(settingsWidget)

        if filename:
            settingsWidget.writeSettings()
            content = self.output(settingsWidget)

            if not content:
                LOGGER.error("No content. Nothing saved.")
                return

            with open(filename, "wt", encoding="utf8", newline="\n") as f:
                f.write(content)

    def preview(self, settingsWidget, previewWidget):
        settings = settingsWidget.getSettings()

        # Save settings
        settingsWidget.writeSettings()

        r = self.output(settingsWidget)

        # Set preview font
        self.preparesTextEditView(previewWidget, settings["Preview"]["PreviewFont"])

        previewWidget.setPlainText(r)

    def preparesTextEditView(self, view, textFont):
        cf = QTextCharFormat()
        f = QFont()
        f.fromString(textFont)
        cf.setFont(f)
        view.setCurrentCharFormat(cf)

    def concatenate(self, item: outlineItem, settings) -> str:
        s = settings
        r = ""

        # Do we include item
        if not item.compile() or s["Content"]["IgnoreCompile"]:
            return ""

        # What do we include
        l = item.level()
        if l >= 0:  # item is not root

            if item.isFolder():
                if not s["Content"]["More"] and s["Content"]["FolderTitle"] or\
                       s["Content"]["More"] and s["Content"]["FolderTitle"][l]:

                    r += self.processTitle(item.title(), l, settings)

            elif item.isText():
                if not s["Content"]["More"] and s["Content"]["TextTitle"] or \
                       s["Content"]["More"] and s["Content"]["TextTitle"][l]:

                    r += self.processTitle(item.title(), l, settings)

                if not s["Content"]["More"] and s["Content"]["TextText"] or \
                       s["Content"]["More"] and s["Content"]["TextText"][l]:

                    r += self.processText(item.text(), settings)

        content = ""

        # Add item children
        last = None
        for c in item.children():

            # Separator
            if last:
                # Between folder
                if last == c.type() == "folder":
                    content += s["Separator"]["FF"]

                elif last == c.type() == "md":
                    content += s["Separator"]["TT"]

                elif last == "folder" and c.type() == "md":
                    content += s["Separator"]["FT"]

                elif last == "md" and c.type() == "folder":
                    content += s["Separator"]["TF"]

            content += self.concatenate(c, settings)

            last = c.type()

        # r += self.processContent(content, settings)
        r += content

        return r

    def processTitle(self, text, level, settings):
        return text + "\n"

    def processText(self, content, settings):
        s = settings["Transform"]

        if s["Dash"]:
            content = content.replace("---", "—")

        if s["Ellipse"]:
            content = content.replace("...", "…")

        if s["Spaces"]:
            o = ""
            while o != content:
                o = content
                content = content.replace("  ", " ")

        if s["DoubleQuotes"]:
            q = s["DoubleQuotes"].split("___")
            s["Custom"].append([True, '"(.*?)"', "{}\\1{}".format(q[0], q[1]), True])

        if s["SingleQuote"]:
            q = s["SingleQuote"].split("___")
            s["Custom"].append([True, "'(.*?)'", "{}\\1{}".format(q[0], q[1]), True])

        for enabled, A, B, reg in s["Custom"]:
            if not enabled:
                continue

            if not reg:
                content = content.replace(A, B)

            else:
                content = re.sub(A, B, content)

        content += "\n"

        return content
